/*
 *icctags-ncl2.c
 *
*
*Copyright (c) 1998 Sun Microsystems, Inc.
*
*                   Rights Reserved
*
*Permission is hereby granted, free of charge, to any person 
*obtaining a copy of this software and associated documentation
*files (the "Software"), to deal in the Software without restrict- 
*ion, including without limitation the rights to use, copy, modify, 
*merge, publish distribute, sublicense, and/or sell copies of the 
*Software, and to permit persons to whom the Software is furnished 
*to do so, subject to the following conditions: 
* 
*The above copyright notice and this permission notice shall be 
*included in all copies or substantial portions of the Software. 
* 
*THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
*OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-
*INFRINGEMENT.  IN NO EVENT SHALL SUN MICROSYSTEMS, INC. BE LIABLE 
*FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
*WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
*FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
*OTHER DEALINGS IN THE SOFTWARE. 
* 
*Except as contained in this notice, the name of Sun MIcrosystems, Inc. 
*shall not be used in advertising or otherwise to promote the 
*sale, use or other dealings in this Software without written 
*authorization from Sun Microsystems Inc. 
*
*
 * Dawn Wallner, Sun Microsystems, 2000
 * This is a test program to  create ICC profile tags
 * independent of any CMM or user interface application.
 * This is example code - not intended to be elegent or efficient, but
 * meant to demonstrate tag access..  You may freely make use of any
 * parts of this code - but it is not guarenteed  or supported.
 * This code is not sanctioned by the ICC or by Sun Microsystems - any
 * errors are the fault of the author.   Please send errors to author
 *
 *
 * Notes:
 * ICC profiles are always stored big-endian.
 * Sample command line executions:
 * icctags outprof.prof    (write a new profile)
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#ifdef _WIN32
#define _LITTLE_ENDIAN
#else
#include <sys/isa_defs.h>
#include <sys/ddi.h>
#endif
#include <math.h>
#include "icc.h"
double  icfixed2double(long val, long type);
double  icfixed2gamma(icUInt16Number);
long  	double2icfixed(double val, long type);
#ifdef _WIN32
typedef unsigned short ushort;
#endif

main(int argc, char **argv)
{
    icInt32Number       rc, ri, gi, bi;
    char		buf[100][5], charstring[5];
    icInt32Number	i, ii, ii2, j, jj, jj2, dumpfile, size, dirptr, 
                        tagdataptr, tempdataptr, perrow;
    FILE		*fd;
    float               f32, ff32;
    icUInt32Number	n32;
    icUInt16Number	n16;
    icUInt8Number	n8;
    icUInt16Number 	val;
    icUInt32Number	ntags;
    icTag		tag[100];
    long		swap(long);
    ushort		swap16(ushort);
    icSignature        	sig;
    icUInt64Number  	tmp64;
    icUInt32Number  	count;

    icHeader		header, *hdr;
    icXYZType		xyz;
    icCurveType 	*curvealloc;
    icCurve  		*curve;
    icDataType  	*datatalloc;
    icData		*datat;
    icDateTimeType  	datetime;
    icTextDescriptionType 	*textdescalloc;
    icTextDescription 	*textdesc;
    icTextType		*textalloc;
    icText		*text;
    icLut16Type		*lut16alloc;
    icLut16		*lut16;
    icLut8Type		*lut8alloc;
    icLut8		*lut8;
    icMeasurementType	measurement;
    icProfileSequenceDescType 	*profseqdescalloc;
    icProfileSequenceDesc	*profseqdesc;
    icInt8Number	*dataptr;
    icSignatureType	tech;
    icCrdInfoType	*crdalloc;
    icCrdInfo		*crd;
    icScreeningType	*screenalloc;
    icScreening		*screen;
    icScreeningData 	*scptr;
    icUcrBgType		*ucrbgalloc;
    icUcrBg		*ucrbg;
    icViewingConditionType	view;
    icNamedColor2Type	*nc2alloc;
    icNamedColor2 	*nc2;
    char		*privatealloc;
    icUInt32Number	*private;
    time_t               clocktime;
    struct tm           *tmdatetime;
    icTag		tagdir;
    icUInt32Number	tblenum, lutnum;
    icTagBase		tagbase;
    div_t		divt;
    icInt8Number        *ptr8, *ptr8save;
    icDescStruct	*descstructalloc, *descstruct;
    

    if ((fd = fopen(argv[1], "wb+")) == NULL) {
      perror("Unable to create file");
      exit(1);
    } 

      /*For example purposes, the data for each tag is hard-coded in each tag case.
	One would normally fill in the tags via an input data file or an interactive
	application, etc.  This example will create a profile with an example of 
	every tag - not a typical or even valid scenerio - but it works for 
	instruction.  icc.h structures will be used.*/

      /*First, indicate the number of tags to be entered in the profile, not counting
	the header.  For this example, we will include one of each = 42.  The the
	size of the header and tag directory can be calculated.  Header = 128 bytes
        and each tag index in the tag directory is 12 bytes. */


      for (i=0; i<4; i++) tagbase.reserved[i] = 0;
      /*      ntags = 42;*/
      ntags = 7;
      size = 128 + 4 + (12 *ntags);             /*header + ntags + tagdir */
      dirptr = 0;
      tagdataptr = size;                                /*point to start of tag data*/


/******************header**********************************/


      hdr = (icHeader *)calloc(sizeof(icHeader), sizeof(icUInt8Number));
      hdr->size = swap((size + 4)); /*update with final profile size*/
      hdr->cmmId = swap((long)0x4b434d53L);             /*'KCMS'*/
      hdr->version = swap((long)0x21000000L);
      hdr->deviceClass = swap((long)0x6d6e7472L);       /*'mntr'*/
      hdr->colorSpace = swap((long)0x52474220L);        /*'RGB '*/
      hdr->pcs = swap((long)0x4c616220L);               /*'Lab '*/
      hdr->cmmId = swap((long)0x4b434d53L);             /*'KCMS'*/
      hdr->version= swap((long)0x21000000L);
      hdr->deviceClass = swap((long)0x6d6e7472L);       /*'mntr'*/
      hdr->colorSpace = swap((long)0x52474220L);        /*'RGB '*/
      hdr->pcs = swap((long)0x4c616220L);               /*'Lab '*/
      /* Get the time from the system */
      clocktime = time(NULL);
      tmdatetime = localtime(&clocktime);

      hdr->date.seconds = (icUInt16Number)swap16((ushort)tmdatetime->tm_sec);
      hdr->date.minutes = (icUInt16Number)swap16((ushort)tmdatetime->tm_min);
      hdr->date.hours = (icUInt16Number)swap16((ushort)tmdatetime->tm_hour);
      hdr->date.day = (icUInt16Number)swap16((ushort)tmdatetime->tm_mday);
      hdr->date.month = (icUInt16Number)swap16((ushort)(tmdatetime->tm_mon+1));
      hdr->date.year = (icUInt16Number)swap16((ushort)(tmdatetime->tm_year + 1900));

      hdr->magic = swap((long)0x61637370L);             /*'acsp'*/
      hdr->platform = swap((long)0x53554e57L);          /*'SUNW'*/
      hdr->flags = swap((long)0x00000000L);             /*Profile not embedded and
                                                        can be used independently*/
      /*maufacturer and model should be registered with the ICC*/
      hdr->manufacturer = swap((long)0x53554e20L);      /*'SUN '*/
      hdr->model = swap((long)0x31393938L);             /*'1998'*/
      hdr->attributes[1] = swap((long)0);
      hdr->attributes[0] = swap((long)9);       /*Transparency, glossy, positive, B&W*/
      hdr->renderingIntent = swap((long)1 << 16);          /*relative colorimetric*/
      hdr->illuminant.X = double2icfixed((double)0.964188, icSigS15Fixed16ArrayType);
      hdr->illuminant.Y = double2icfixed((double)1.000, icSigS15Fixed16ArrayType);
      hdr->illuminant.Z = double2icfixed((double)0.824890, icSigS15Fixed16ArrayType);
      hdr->creator = swap((long)0x53554e57L);           /*'SUNW'*/
      if (fwrite(hdr, sizeof(icHeader), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }
      fseek(fd, sizeof(icHeader), SEEK_SET);
      ntags = swap((long)ntags);
      if (fwrite(&ntags, sizeof(icUInt32Number), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }
      dirptr =  sizeof(icHeader) + 4;          /*point to start of tag directory*/
      free(hdr);


/**************************desc - profile description tag******************/


      tagdir.sig =  swap((long)0x64657363L);                 /*'desc'*/
      tagdir.offset =  swap((long)tagdataptr);    
      /*tagbase + ascii count + data, UniCode code + count scriptCode code + count.
       *The Unicode desc to be entered is 48 characters */
      /*see below for detail on the size calculations*/
      size = sizeof(tagbase) +
                sizeof(icUInt32Number) + 36 +
                2*sizeof(icUInt32Number) +
                sizeof(icUInt16Number) + sizeof(icUInt8Number) + 67;

      ptr8 = (char *)calloc(130, sizeof(icUInt8Number));
      ptr8save = ptr8;
      printf("tag size = %d\n",size);
      tagdir.size =  swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        } 
      dirptr += sizeof(tagdir);                 /*keep this pointed to the end of the          
                                                  tag directory*/
      tagbase.sig =  swap((long)0x64657363L);                /*'desc'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }   
      tagdataptr +=sizeof(icTagBase);

      /* Write the ASCII data  (48 + 4 bytes)*/
      n32 = swap(36);
      memcpy(ptr8, &n32, sizeof(icUInt32Number));
      ptr8 += sizeof(icUInt32Number);
      strncpy(ptr8,"This is the profile description tag",36); 
      ptr8 += 36;

      /* Write in the UniCode data (8 bytes minimum) */
      n32 = 0;
      memcpy(ptr8, &n32, sizeof(icUInt32Number));  /*Unicode language code */
      ptr8 += sizeof(icUInt32Number);
      memcpy(ptr8, &n32, sizeof(icUInt32Number));  /*Unicode description count*/
      ptr8 += sizeof(icUInt32Number);

      /* Write in the ScriptCode data (70 bytes minimum)*/
      n16 = swap16(0);
      n8=0;
      memcpy(ptr8, &n16, sizeof(icUInt16Number));  /*script code language code */
      ptr8 += sizeof(icUInt16Number);
      memcpy(ptr8, &n8, sizeof(icUInt8Number));  /*scriptcode count */
      ptr8 += sizeof(icUInt8Number);
      memcpy(ptr8, &n8, 67);                /*required 67 bytes of 0 */
      ptr8 += 67;    
      n32 = 130;
      if (fwrite(ptr8save, n32, 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }


      /*update the final size of profile in the header*/
      tagdataptr += size - sizeof(tagbase);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }
      free(ptr8save);
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;


 
/********************whitepoint wtpt tag******************************/


     tagdir.sig = swap((long)0x77747074L);                 /*'wtpt'*/
      tagdir.offset = swap((long)tagdataptr);
      size = sizeof(tagbase) + sizeof(icXYZNumber);
      printf("tag size = %d\n",size);
      tagdir.size = swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        } 



      dirptr += sizeof(tagdir);                 /*keep this pointed to the end of the 
						  tag directory*/
      tagbase.sig = swap((long)0x58595a20L);                /*'XYZ '*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }   
      tagdataptr +=sizeof(tagbase);
      xyz.data.data[0].X = double2icfixed (0.964188, icSigS15Fixed16ArrayType);
      xyz.data.data[0].Y = double2icfixed (1.000000, icSigS15Fixed16ArrayType);
      xyz.data.data[0].Z = double2icfixed (0.824875, icSigS15Fixed16ArrayType);
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&xyz.data, sizeof(icXYZNumber), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }

      /*update the final size of profile in the header*/
      tagdataptr += sizeof(icXYZNumber);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;



 

/**********************cprt - copyright tag*******************************/


     tagdir.sig =  swap((long)0x63707274L);                    /*cprt'*/
      tagdir.offset =  swap((long)tagdataptr);
      /*The string to be entered is 34 characters, including ending space, plus base */
      size = sizeof(tagbase) + 34;
      textalloc = (icTextType *)calloc(size, sizeof(icUInt8Number));
      text = &textalloc->data;
      printf("tag size = %d\n",size);
      tagdir.size =  swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        } 
      dirptr += sizeof(tagdir);                 /*keep this pointed to the end of the 
						  tag directory*/
      tagbase.sig =  swap((long)0x74657874L);           /*'text'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }   
      tagdataptr +=sizeof(icTagBase);
      strncpy(text->data,"Copyright:  Sun Microsystems 1998",34);

      if (fwrite(text, 34, 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }

      /*update the final size of profile in the header*/
      tagdataptr += size - sizeof(tagbase);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }
      free(textalloc);
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;



/*******************psd0 PostScript color rendering dictionary 0 tag*************************/



      tagdir.sig =  swap((long)0x70736430L);                    /*psd0'*/
      tagdir.offset =  swap((long)tagdataptr);
      /*The string to be entered is 34 characters, including ending space, plus base */
      size = sizeof(tagbase) + 18;
      datatalloc = (icDataType *)calloc(size, sizeof(icUInt8Number));
      memset(datatalloc, 0, size);
      datat = &datatalloc->data;
      printf("tag size = %d\n",size);
      tagdir.size =  swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        } 
      dirptr += sizeof(tagdir);                 /*keep this pointed to the end of the           tag directory*/
      tagbase.sig =  swap((long)0x64617461L);           /*'data'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }   
      tagdataptr +=sizeof(icTagBase);
      datat->dataFlag = 0;                      /*indicates ascii data to follow*/
      strncpy(datat->data,"PS2 CRD 0 tag",14);

      if (fwrite(datat, 18, 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }

      /*update the final size of profile in the header*/
      tagdataptr += size - sizeof(tagbase);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }
      memset(datatalloc, 0, size);
      free(datatalloc);
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;




/**************************ps2i PostScript Level 2 rendering intent tag***********************/


      tagdir.sig =  swap((long)0x70733269L);                    /*ps2i'*/
      tagdir.offset =  swap((long)tagdataptr);
      /*The string to be entered is 34 characters, including ending space, plus base */
      size = sizeof(tagbase) + 26;
      datatalloc = (icDataType *)calloc(size, sizeof(icUInt8Number));
      memset(datatalloc, 0, size);
      datat = &datatalloc->data;
      printf("tag size = %d\n",size);
      tagdir.size =  swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        } 
      dirptr += sizeof(tagdir);                 /*keep this pointed to the end of the           tag directory*/
      tagbase.sig =  swap((long)0x64617461L);           /*'data'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }   
      tagdataptr +=sizeof(icTagBase);
      datat->dataFlag = 0;                      /*indicates ascii data to follow*/
      strncpy(datat->data,"RelativeColorimetric ",22);

      if (fwrite(datat, 26, 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }

      /*update the final size of profile in the header*/
      tagdataptr += size - sizeof(tagbase);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }
      memset(datatalloc, 0, size);
      free(datatalloc);
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;



/*************************ncl2  Named color tag***************************/




      tagdir.sig =  swap((long)0x6e636c32L);                    /*ncl2'*/
      tagdir.offset =  swap((long)tagdataptr);

      size = sizeof(tagbase) +
        3 * sizeof(icUInt32Number) +    /*vendor, count, # device coords*/
        (64 *sizeof(icUInt8Number)) +   /*2*32 - prefix, suffix arrrays */
        (64 * sizeof(icUInt8Number))+   /*root name array*/
        (12 * sizeof(icUInt16Number));  /*2 colors * 3 coords * 2 sets */

      nc2alloc = (icNamedColor2Type *)calloc(size, sizeof(icUInt8Number));
      memset(nc2alloc, 0, size);
      nc2 = &nc2alloc->ncolor;
      dataptr = (char *)nc2->data;
      
      printf("tag size = %d\n",size);
      tagdir.size =  swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        } 
      dirptr += sizeof(tagdir);                 /*keep this pointed to the end of the           tag directory*/
      tagbase.sig =  swap((long)0x64617461L);           /*'data'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }   
      tagdataptr +=sizeof(icTagBase);
      nc2->vendorFlag =  swap((long)0x00040000);                /*vendor specific flag-upper 16bi
ts*/
      nc2->count =  swap((long)2);                              /* number of colors described*/
      nc2->nDeviceCoords =  swap((long)3);                      /* # coords for each color*/
      strncpy(nc2->prefix, "light", 32);
      strncpy(nc2->suffix, "ish", 32);
      /* color 1 */
      strncpy(dataptr, "red", 32);              /* root color name */
      dataptr += 32;
      n16 = swap16((ushort)255);
      memcpy(dataptr, &n16, sizeof(icUInt16Number));    /*the 3 PCS coords */
      dataptr += sizeof(icUInt16Number);
      n16 =  swap16((ushort)0);
      memcpy(dataptr, &n16, sizeof(icUInt16Number));
      dataptr += sizeof(icUInt16Number);
      n16 =  swap16((ushort)0);
      memcpy(dataptr, &n16, sizeof(icUInt16Number));
      dataptr += sizeof(icUInt16Number);

      n16 =  swap16((ushort)128);
      memcpy(dataptr, &n16, sizeof(icUInt16Number));    /*the 3 device coords */
      dataptr += sizeof(icUInt16Number);
      n16 =  swap16((ushort)3);
      memcpy(dataptr, &n16, sizeof(icUInt16Number));
      dataptr += sizeof(icUInt16Number);
      n16 =  swap16((ushort)4);
      memcpy(dataptr, &n16, sizeof(icUInt16Number));
      dataptr += sizeof(icUInt16Number);

      /* color 2 */
      strncpy(dataptr, "blue", 32);     /* root color name */
      dataptr += 32;
      n16 =  swap16((ushort)0);
      memcpy(dataptr, &n16, sizeof(icUInt16Number));    /*the 3 PCS coords */
      dataptr += sizeof(icUInt16Number);
      n16 =  swap16((ushort)0);
      memcpy(dataptr, &n16, sizeof(icUInt16Number));
      dataptr += sizeof(icUInt16Number);
      n16 =  swap16((ushort)255);
      memcpy(dataptr, &n16, sizeof(icUInt16Number));
      dataptr += sizeof(icUInt16Number);

      n16 =  swap16((ushort)10);
      memcpy(dataptr, &n16, sizeof(icUInt16Number));    /*the 3 device coords */
      dataptr += sizeof(icUInt16Number);
      n16 =  swap16((ushort)9);
      memcpy(dataptr, &n16, sizeof(icUInt16Number));
      dataptr += sizeof(icUInt16Number);
      n16 =  swap16((ushort)200);
      memcpy(dataptr, &n16, sizeof(icUInt16Number));
      dataptr += sizeof(icUInt16Number);


      if (fwrite(nc2, size - sizeof(tagbase), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }

      /*update the final size of profile in the header*/
      tagdataptr += size - sizeof(tagbase);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }
      memset(nc2alloc, 0, size);
      free(nc2alloc);
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;



/*************************crdi CRD info tag***************************/


      tagdir.sig =  swap((long)0x63726469L);                    /*crdi'*/
      tagdir.offset = swap((long)tagdataptr);

      size = sizeof(tagbase) +
	sizeof(icUInt32Number) +	/*count*/
        5 * sizeof(icUInt32Number) +    /*5 sets of character counts*/
        136;                            /*5 strings of characters totaled */

      crdalloc = (icCrdInfoType *)calloc(size, sizeof(icUInt8Number));
      crd = &crdalloc->info;
      dataptr = crd->data;
      printf("tag size = %d\n",size);
      tagdir.size = swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        } 
      dirptr += sizeof(tagdir);                 /*keep this pointed to the end of the           tag directory*/
      tagbase.sig = swap((long)0x63726469L);            /*'crdi'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }   
      tagdataptr +=sizeof(icTagBase);
      n32 = swap((long)24);
      memcpy(dataptr, &n32, sizeof(icUInt32Number));
      dataptr += sizeof(icUInt32Number);
      strncpy(dataptr, "PostScript product name", 24);
      dataptr += 24;
      n32 = swap((long)28);
      memcpy(dataptr, &n32, sizeof(icUInt32Number));
      dataptr += sizeof(icUInt32Number);
      strncpy(dataptr, "Rendering intent 0 CRD name", 28);
      dataptr += 28;
      n32 = swap((long)28);
      memcpy(dataptr, &n32, sizeof(icUInt32Number));
      dataptr += sizeof(icUInt32Number);
      strncpy(dataptr, "Rendering intent 1 CRD name", 28);
      dataptr += 28;
      n32 = swap((long)28);
      memcpy(dataptr, &n32, sizeof(icUInt32Number));
      dataptr += sizeof(icUInt32Number);
      strncpy(dataptr, "Rendering intent 2 CRD name", 28);
      dataptr += 28;
      n32 = swap((long)28);
      memcpy(dataptr, &n32, sizeof(icUInt32Number));
      dataptr += sizeof(icUInt32Number);
      strncpy(dataptr, "Rendering intent 3 CRD name", 28);
      dataptr += 28;

 
      if (fwrite(crd->data, size - sizeof(tagbase), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }

      /*update the final size of profile in the header*/
      tagdataptr += size - sizeof(tagbase);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }
      free(crdalloc);
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;



      exit(0);
    }


/******************Support Functions*********************/


/*
 * Routine to byte swap a long, just returns on big-endian
 */
long
swap(long value)
{
#ifdef _LITTLE_ENDIAN
    char	*ptr, c;

    ptr = (char *)&value;
    c = *(ptr+1);
    *(ptr+1) = *(ptr+2);
    *(ptr+2) = c;

    c = *ptr;
    *ptr = *(ptr+3);
    *(ptr+3) = c;
#endif _LITTLE_ENDIAN

    return(value);
}

/*
 * Routine to byte swap a short, just returns on big-endian
 */
ushort
swap16(ushort value)
{
#ifdef _LITTLE_ENDIAN
    char        *ptr, c;

    ptr = (char *)&value;
    c = *(ptr+1);
    *(ptr+1) = *(ptr);
    *ptr = c;

#endif _LITTLE_ENDIAN

    return(value);
}


/* Convert an 8.8 unsigned fixed type to double */
double
icfixed2gamma(icUInt16Number val)
{
    double      retval;
    ushort a, b;

    val = swap16((ushort)val);
    retval = (double) (val/256.0);
    return (retval);
}

/*
 * Convert IC fixed point to a double
 */
double
icfixed2double(long val, long type)
{
    double              retval;
    val = swap((long)val);
    switch(type) {
    case icSigS15Fixed16ArrayType :
        retval = (double) (val / 65536.0);
        break;
    case icSigU16Fixed16ArrayType :
        retval = (double)(val/65536.0);
        break;
    default :
        retval  = -1.0;
        break;
    }
    return(retval);
}
/*
 * Convert double to an IC fixed point number
 * is not accurate enough for boundary conditions
 */
long
double2icfixed(double val, long type)
{
    long                retval;
    short               a;
    ushort      b, c;

    switch(type) {
    case icSigS15Fixed16ArrayType :
        a = (short)(val);
        b = (ushort)((val - a) * 65536.0);
        retval = (long) ((a <<16) | b);
        break;
    case icSigU16Fixed16ArrayType :
        c = (ushort)(val);
        b = (ushort)((val - c) * 65536.0);
        retval = (unsigned long)((c <<16) | b);
        break;
    default :
        retval  = -1.0;
        break;
    }
    retval = swap((long)retval);
    return(retval);
}
