/*
 * icctags.c
 *
*
*Copyright (c) 1999 Sun Microsystems, Inc.
*
*                   Rights Reserved
*
*Permission is hereby granted, free of charge, to any person 
*obtaining a copy of this software and associated documentation
*files (the "Software"), to deal in the Software without restrict- 
*ion, including without limitation the rights to use, copy, modify, 
*merge, publish distribute, sublicense, and/or sell copies of the 
*Software, and to permit persons to whom the Software is furnished 
*to do so, subject to the following conditions: 
* 
*The above copyright notice and this permission notice shall be 
*included in all copies or substantial portions of the Software. 
* 
*THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
*OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-
*INFRINGEMENT.  IN NO EVENT SHALL SUN MICROSYSTEMS, INC. BE LIABLE 
*FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
*WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
*FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
*OTHER DEALINGS IN THE SOFTWARE. 
* 
*Except as contained in this notice, the name of Sun MIcrosystems, Inc. 
*shall not be used in advertising or otherwise to promote the 
*sale, use or other dealings in this Software without written 
*authorization from Sun Microsystems Inc. 
*
*
 * Dawn Wallner, Sun Microsystems, 1998
 * This is a test program to dump and/or create ICC profile tags
 * independent of any CMM or user interface application.
 * This is example code - not intended to be elegent or efficient, but
 * meant to demonstrate tag access..  You may freely make use of any
 * parts of this code - but it is not guarenteed  or supported.
 * This code is not sanctioned by the ICC or by Sun Microsystems - any
 * errors are the fault of the author.   Please send errors to author
 *
 *
 * Notes:
 * ICC profiles are always stored big-endian.
 * Sample command line executions:
 * icctags -r monitor.prof    (dump the tags in the profile called monitor.prof)
 * icctags -rv printer.prof   (also dump all the lut tags - this can be large)
 * icctags -w outprof.prof    (write a new profile)
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#ifdef _WIN32
#define _LITTLE_ENDIAN
#else
#include <sys/isa_defs.h>
#include <sys/ddi.h>
#endif
#include <time.h>
#include <math.h>
#include "icc.h"
double  icfixed2double(long val, long type);
double  icfixed2gamma(icUInt16Number);
long  	double2icfixed(double val, long type);
#ifdef _WIN32
typedef unsigned short ushort;
#endif

void 	print_luttag(FILE *fd, icTag tag, icUInt32Number verbose,
		     char buf[5]);

void print_usage()
{
	fprintf(stderr, 
		"usage : gettags [-rwv] [readprofileanme | writeprofilename] \n");
	fprintf(stderr, 
		"Use the -r to read and dump a profile, the -w to write a profile, and -v option to dump all the LUT tag data.\n");
}


main(int argc, char **argv)
{
    icInt32Number	readfile, writefile, verbose, rc, ri, gi, bi;
    char		buf[100][5], charstring[5];
    icInt32Number	i, ii, ii2, ii3, ii4, j, jj, jj2, 
                        channelvals[10], dumpfile, size, dirptr, 
                        tagdataptr, tempdataptr, perrow;
    FILE		*fd;
    float               f32;
    icUInt32Number	n32;
    icUInt16Number	n16;
    icUInt8Number	n8;
    icUInt16Number 	val;
    icUInt32Number	ntags;
    icTag		tag[100];
    long		swap(long);
    ushort		swap16(ushort);
    icSignature        	sig;
    icUInt64Number  	tmp64;
    icUInt32Number  	count;

    icHeader		header, *hdr;
    icXYZType		xyz;
    icCurveType 	*curvealloc;
    icCurve  		*curve;
    icDataType  	*datatalloc;
    icData		*datat;
    icDateTimeType  	datetime;
    icTextDescriptionType 	*textdescalloc;
    icTextDescription 	*textdesc;
    icTextType		*textalloc;
    icText		*text;
    icLut16Type		*lut16alloc;
    icLut16		*lut16;
    icLut8Type		*lut8alloc;
    icLut8		*lut8;
    icMeasurementType	measurement;
    icProfileSequenceDescType 	*profseqdescalloc;
    icProfileSequenceDesc	*profseqdesc;
    icInt8Number	*dataptr;
    icSignatureType	tech;
    icCrdInfoType	*crdalloc;
    icCrdInfo		*crd;
    icScreeningType	*screenalloc;
    icScreening		*screen;
    icScreeningData 	*scptr;
    icUcrBgType		*ucrbgalloc;
    icUcrBg		*ucrbg;
    icViewingConditionType	view;
    icNamedColor2Type	*nc2alloc;
    icNamedColor2 	*nc2;
    icChromaticityType	*chromaalloc;
    icChromaticity 	*chroma;
    char		*privatealloc;
    icUInt32Number	*private;
    time_t               clocktime;
    struct tm           *tmdatetime;
    icTag		tagdir;
    icUInt32Number	tblenum, lutnum;
    icTagBase		tagbase;
    div_t		divt;
    icInt8Number        *ptr8, *ptr8save;
    icDescStruct	*descstructalloc, *descstruct;
    icDeviceSettingsType *settingstype;
    icUInt32Number	ptr32;
    icSettingsData    	*settingsdata;
    icResponseCurveSet16Type    *responsetype;
    icResponse		*responsedata;
    icResponseCurveSet16Type    *respcurvetype;
    icResponse          *response;

    verbose = 0;
    readfile = 0;
    writefile = 0;
		if (argc>1) {
			if (argv[1][0]=='-') {
				if (strchr(argv[1], 'r'))
					readfile = 1;
				if (strchr(argv[1], 'w'))
					writefile = 1;
				if (strchr(argv[1], 'v'))
					verbose = 1;
				if (strchr(argv[1], '?')) {
					print_usage();
					exit(1);
				}
			}
		}
		else {
			print_usage();
			exit(1);
		}


    if (readfile) {
      if ((fd = fopen(argv[2], "rb")) == NULL) {
	perror("Bad file name");
	exit(1);
      }
      dumpfile = 1;
    } else if (writefile) {
      if ((fd = fopen(argv[2], "wb+")) == NULL) {
	perror("Unable to create file");
	exit(1);
      } 
      dumpfile = 0;
   }else {
      printf("valid arguments for gettags are <r> <input file name> or \n");
      printf("<w> <output filename> and <v> for verbose\n"); 
      exit(1);
    }

  if (dumpfile) {

      if (fread(&header, sizeof(icHeader), 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}

/*************************header*************************/

      printf("Size in bytes = %d\n", swap((long)header.size));
      memcpy(charstring, &header.cmmId, 4);
      printf("CMM Id = %.4s\n", charstring);
      printf("Version number = 0x%x\n", swap((long)header.version));

      switch(swap((long)header.deviceClass)) {
      case icSigInputClass :
        printf("deviceClass = input\n");
        break;
      case icSigDisplayClass :
        printf("deviceClass = display\n");
        break;
      case icSigOutputClass :
        printf("deviceClass = output\n");
        break;
      case icSigLinkClass :
        printf("deviceClass = link\n");
        break;
      case icSigAbstractClass :
        printf("deviceClass = abstract\n");
        break;
      case icSigColorSpaceClass :
        printf("deviceClass = colorspace\n");
        break;
      default :
        printf("Unknown\n");
        break;
    
    }

      memcpy(charstring, &header.colorSpace, 4);
      printf("colorspace = %.4s\n", charstring);
      memcpy(charstring, &header.pcs, 4);
      printf("profile connection space = %.4s\n", charstring);
      printf("date = %d/%d/%d,  ",swap16((ushort)header.date.day),
	swap16((ushort)header.date.month), swap16((ushort)header.date.year));
      printf("time = %d:%d:%d\n",swap16((ushort)header.date.hours),
	swap16((ushort)header.date.minutes),swap16((ushort)header.date.seconds));

      memcpy(charstring, &header.magic, 4);
      printf("magic number = %.4s\n", charstring);

      switch(swap((long)header.platform)) {
      case icSigMacintosh :
        printf("platform = Macintosh\n");
        break;
      case icSigMicrosoft :
        printf("platform = Microsoft\n");
        break;
      case icSigSolaris :
        printf("platform = Solaris\n");
        break;
      case icSigSGI :
        printf("platform = SGI\n");
        break;
      case icSigTaligent :
        printf("platform = Taligent\n");
        break;
      default :
        printf("Unknown\n");
        break;
      }

      if(swap((long)header.flags) && icEmbeddedProfileTrue)
        printf("Embedded profile.\n");
      else
        printf("Non-embedded profile\n");

      if(swap((long)header.flags) && icUseWithEmbeddedDataOnly) {
        printf("If this profile is embedded, ");
        printf("it is not allowed to strip it out");
        printf(" and use it independently.\n");
      } else
        printf("OK to strip embedded profile out and use independently\n");

      memcpy(charstring, &header.manufacturer, 4);
      printf("manufacturer = %.4s\n", charstring);

      memcpy(charstring, &header.model, 4);
      printf("model = %.4s\n", charstring);

      if ((swap((long)header.attributes[0]) & 0x00000001) == 1)
	printf("Attributes = transparency, ");
      else 
	printf("Attributes = reflective, ");
      if ((swap((long)header.attributes[0]) & 0x00000002) == 2) 
	printf("matte, ");
      else
	printf("glossy, ");
      if ((swap((long)header.attributes[0]) & 0x00000004) == 4) 
	printf("negative, ");
      else
	printf("positive, ");
      if ((swap((long)header.attributes[0]) & 0x00000008) == 8) 
	printf("black & white\n");
      else
	printf("color\n");


      header.renderingIntent =  
	swap((long)header.renderingIntent & 0x00030000);
      switch (header.renderingIntent) {
      case 0 :
        printf("rendering intent = Perceptual\n");
        break;
      case 65536 :
        printf("rendering intent = Relative Colorimetric\n");
        break;
      case 131072 :
        printf("rendering intent = Saturation\n");
        break;
      case 196608 :
        printf("rendering intent = Absolute Colorimetric\n");
        break;
      default :
        printf("Unknown\n");
        break;
      }					    
      printf("Illuminat X=%f Y=%f Z=%f\n", 
                icfixed2double(header.illuminant.X, icSigS15Fixed16ArrayType),
                icfixed2double(header.illuminant.Y, icSigS15Fixed16ArrayType),
                icfixed2double(header.illuminant.Z, icSigS15Fixed16ArrayType));

      memcpy(charstring, &header.creator, 4);
      printf("creator = %.4s\n", charstring);

      printf("\n\n");



/***********************tag directory*****************************/

      
      /* Seek past the header */
      fseek(fd, sizeof(icHeader), SEEK_SET);

      /* Read the count and swap if needed */
      if (fread(&ntags, sizeof(ntags), 1, fd) != 1) {
	printf("error reading file\n");
	exit(1);
      }
      ntags = swap((long)ntags);
      /* Get the tags one by one and print the values */
      for (i=0; i<ntags; i++) {
	if (fread(&tag[i], sizeof(icTag), 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	buf[i][4] = 0;
	/*
	 * Copy this to a string here means that it appears
	 * correctly on Intel.
	 */
        memcpy(buf[i], &tag[i].sig, 4);

	/* Swap the data */
	tag[i].sig = swap(tag[i].sig);
	tag[i].offset = swap(tag[i].offset);
	tag[i].size = swap(tag[i].size);

      }
      printf("Number of tags in this profile = %d\n", ntags);
      /* 43 current tags plus private tags, so allow for up to 100 tags */
      for (i=0; i<ntags; i++) {
	switch(tag[i].sig) { 


/***********************icSigAToB0Tag*****************************/


      case 0x41324230L:  /* 'A2B0' */
	print_luttag(fd, tag[i], verbose, buf[i]);
	break;



/***********************icSigAToB1Tag*****************************/


      case 0x41324231L:  /* 'A2B1' */
	print_luttag(fd, tag[i], verbose, buf[i]);
	break;



/***********************icSigAToB2Tag*****************************/


      case 0x41324232L:  /* 'A2B2' */ 
	print_luttag(fd, tag[i], verbose, buf[i]);
	break;



/***********************icSigBlueColorantTag*****************************/


      case 0x6258595AL:  /* 'bXYZ' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	if (fread(&xyz, sizeof(icXYZType), 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
        printf("XYZ type\n");
	printf("X=%f, Y=%f, Z=%f\n",
	       icfixed2double(xyz.data.data[0].X,
			      icSigS15Fixed16ArrayType),
	       icfixed2double(xyz.data.data[0].Y,
			      icSigS15Fixed16ArrayType),
	       icfixed2double(xyz.data.data[0].Z,
			      icSigS15Fixed16ArrayType));

        printf("\n\n");
	break;


/*************************icSigBlueTRCTag***************************/


      case 0x62545243L:  /* 'bTRC' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	curvealloc = (icCurveType *)calloc(tag[i].size, sizeof(icUInt8Number));
	if (fread(curvealloc,  tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	curve = &curvealloc->curve;
        curve->count = swap((long)curve->count);
        printf("Curve type, curve count = %d\n",curve->count );
            /* Linear */
            if (curve->count == 0) {
                printf("Count = 0 Curve is linear\n");
             }
            /* Gamma value */
            else if (curve->count == 1) {
                printf("Count = 1 Curve is a gamma of %f\n",
                       icfixed2gamma((icUInt16Number)curve->data[0]));
            }
            /* Beginning/end points of a line */
            else if (curve->count == 2) {
                printf("Count = 2 Line Start = %d End = %d\n",
                      (icUInt16Number)swap16((ushort)curve->data[0]), 
			(icUInt16Number)swap16((ushort)curve->data[1])); 
            } else {

	    /* Just a plain old curve */
	    for (j=0; j<curve->count; j++) {
	      printf("%d ", (icUInt16Number)swap16((ushort)curve->data[j]));
	      if (!(j%16) && (j!=0))
                printf("\n");
	    }
	    }

        printf("\n\n");
	free(curvealloc);
	    break;


/************************icSigBToA0Tag*****************************/


      case 0x42324130L:  /* 'B2A0' */
	print_luttag(fd, tag[i], verbose, buf[i]);
	break;



/**********************icSigBToA1Tag******************************/


      case 0x42324131L:  /* 'B2A1' */
	print_luttag(fd, tag[i], verbose, buf[i]);
	break;



/*************************icSigBToA2Tag****************************/


      case 0x42324132L:  /* 'B2A2' */
	print_luttag(fd, tag[i], verbose, buf[i]);
	break;



/**********************icSigCalibrationDateTimeTag******************************/


      case 0x63616c74L:  /* 'calt' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	if (fread(&datetime, sizeof(icDateTimeType), 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
        printf("Date type\n");
        printf("Date = %d/%d/%d Time = %d:%d:%d\n",
               swap16((ushort)datetime.date.month),
               swap16((ushort)datetime.date.day),
               swap16((ushort)datetime.date.year),
               swap16((ushort)datetime.date.hours),
               swap16((ushort)datetime.date.minutes),
               swap16((ushort)datetime.date.seconds));

        printf("\n\n");
	break;


/*********************icSigCharTargetTag*******************************/


      case 0x74617267L:  /* 'targ' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	textalloc = (icTextType *)calloc(tag[i].size, sizeof(icUInt8Number));
	if (fread(textalloc, tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	text = &textalloc->data;
        printf("Text type\n");
        printf("%s", text->data);
        printf("\n\n");
	free(textalloc);
	break; 


/***********************icSigCopyrightTag*****************************/


      case 0x63707274L:  /* 'cprt' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	textalloc = (icTextType *)calloc(tag[i].size, sizeof(icUInt8Number));
	if (fread(textalloc, tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	text = &textalloc->data;
        printf("Text type\n");
        printf("%s", text->data);
        printf("\n\n");
	free(textalloc);
	break;


/***********************icSigDeviceMfgDescTag*****************************/


      case 0x646D6E64L:  /* 'dmnd' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	textdescalloc = (icTextDescriptionType *)calloc(tag[i].size, sizeof(icUInt8Number));
	if (fread(textdescalloc, tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	textdesc = &textdescalloc->desc;
        printf("Text description \n");
        printf("%s\n", textdesc->data);

        printf("\n\n");
	free(textdescalloc);
	break;


/**********************icSigDeviceModelDescTag******************************/


      case 0x646D6464L:  /* 'dmdd' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	textdescalloc = (icTextDescriptionType *)calloc(tag[i].size, sizeof(icUInt8Number));
	if (fread(textdescalloc, tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	textdesc = &textdescalloc->desc;
        printf("Text description \n");
        printf("%s\n", textdesc->data);

        printf("\n\n");
	free(textdescalloc);
	break;


/***********************icSigGamutTag******************************/


       case 0x67616D74L:  /* 'gamt ' */
	print_luttag(fd, tag[i], verbose, buf[i]);
	break;



/**********************icSigGrayTRCTag******************************/


      case 0x6b545243L:  /* 'kTRC' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	curvealloc = (icCurveType *)calloc(tag[i].size, sizeof(icUInt8Number));
	if (fread(curvealloc,  tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	curve = &curvealloc->curve;
        curve->count = swap((long)curve->count);
        printf("Curve type, curve count = %d\n",curve->count );
            /* Linear */
            if (curve->count == 0) {
                printf("Count = 0 Curve is linear\n");
            }
            /* Gamma value */
            else if (curve->count == 1) {
                printf("Count = 1 Curve is a gamma of %f\n",
                       icfixed2gamma((icUInt16Number)curve->data[0]));
            }
            /* Beginning/end points of a line */
            else if (curve->count == 2) {
                printf("Count = 2 Line Start = %d End = %d\n",
                      (icUInt16Number)curve->data[0], (icUInt16Number)curve->data[1]); 
            } else {

	    /* Just a plain old curve */
	    for (j=0; j<curve->count; j++) {
	      printf("%d ", (icUInt16Number)curve->data[j]);
	      if (!(j%16) && (j!=0))
                printf("\n");
	    }
	    }

        printf("\n\n");
	free(curvealloc);

	break;


/**********************icSigGreenColorantTag*******************************/


      case 0x6758595AL:  /* 'gXYZ' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	if (fread(&xyz, sizeof(icXYZType), 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
        printf("XYZ type\n");
	printf("X=%f, Y=%f, Z=%f\n",
	       icfixed2double(xyz.data.data[0].X,
			      icSigS15Fixed16ArrayType),
	       icfixed2double(xyz.data.data[0].Y,
			      icSigS15Fixed16ArrayType),
	       icfixed2double(xyz.data.data[0].Z,
			      icSigS15Fixed16ArrayType));
        printf("\n\n");
	break;


/*********************icSigGreenTRCTag*******************************/


      case 0x67545243L:  /* 'gTRC' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	curvealloc = (icCurveType *)calloc(tag[i].size, sizeof(icUInt8Number));
	if (fread(curvealloc,  tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	curve = &curvealloc->curve;
	curve->count = swap((long)curve->count);
        printf("Curve type, curve count = %d\n",curve->count );
            /* Linear */
            if (curve->count == 0) {
                printf("Count = 0 Curve is linear\n");
            }
            /* Gamma value */
            else if (curve->count == 1) {
                printf("Count = 1 Curve is a gamma of %f\n",
                       icfixed2gamma((icUInt16Number)curve->data[0]));
            }
            /* Beginning/end points of a line */
            else if (curve->count == 2) {
                printf("Count = 2 Line Start = %d End = %d\n",
                      (icUInt16Number)swap16((ushort)curve->data[0]), 
		       (icUInt16Number)swap16((ushort)curve->data[1])); 
            } else {

	    /* Just a plain old curve */
	    for (j=0; j<curve->count; j++) {
	      printf("%d ", (icUInt16Number)swap16((ushort)curve->data[j]));
	      if (!(j%16) && (j!=0))
                printf("\n");
	    }
	    }

        printf("\n\n");
	free(curvealloc);

	break;


/***********************icSigLuminanceTag******************************/


      case 0x6C756d69L:  /* 'lumi' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	if (fread(&xyz, sizeof(icXYZType), 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
        printf("XYZ type\n");
	printf("X=%f, Y=%f, Z=%f\n",
	       icfixed2double(xyz.data.data[0].X,
			      icSigS15Fixed16ArrayType),
	       icfixed2double(xyz.data.data[0].Y,
			      icSigS15Fixed16ArrayType),
	       icfixed2double(xyz.data.data[0].Z,
			      icSigS15Fixed16ArrayType));
        printf("\n\n");
	break;


/*********************icSigMeasurementTag*******************************/


      case 0x6D656173L:  /* 'meas' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	if (fread(&measurement, sizeof(icMeasurementType), 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	printf("Measurement Type\n");

	switch(swap((long)measurement.measurement.stdObserver)) {
	case icStdObsUnknown :
	  printf("Standard Observer = Unknown\n");
	  break;
	case icStdObs1931TwoDegrees :
	  printf("Standard Observer = 1931 two degrees\n");
	  break;
	case icStdObs1964TenDegrees :
	  printf("Standard Observer = 1964 ten degrees\n");
	  break;
	default :
	  printf("Unknown\n");
	  break;
	}
	printf("XYZ backing:  ");
	printf("X=%f, Y=%f, Z=%f\n",
	       icfixed2double(measurement.measurement.backing.X,
			      icSigS15Fixed16ArrayType),
	       icfixed2double(measurement.measurement.backing.Y,
			      icSigS15Fixed16ArrayType),
	       icfixed2double(measurement.measurement.backing.Z,
			      icSigS15Fixed16ArrayType));
	printf("Geometry Type = ");
	switch(swap((long)measurement.measurement.geometry)) {
	case icGeometryUnknown :
	  printf("Unknown\n");
	  break;
	case icGeometry045or450 :
	  printf("0/45 or 45/0\n");
	  break;
	case icGeometry0dord0 :
	  printf("0/d or d/0\n");
	  break;
	default :
	  printf("Unknown\n");
	  break;
	}
        printf("Flare percent %f  ",icfixed2double(measurement.measurement.flare, 
                         icSigU16Fixed16ArrayType));

	printf("Illuminant =");
	switch(swap((long)measurement.measurement.illuminant)) {
	case icIlluminantUnknown  :
	  printf("Unknown\n");
	  break;
	case icIlluminantD50  :
	  printf("D50\n");
	  break;
	case icIlluminantD65  :
	  printf("D65\n");
	  break;
	case icIlluminantD93  :
	  printf("D93\n");
	case icIlluminantF2  :
	  printf("F2\n");
	  break;
	case icIlluminantD55  :
	  printf("D55\n");
	  break;
	case icIlluminantA  :
	  printf("A\n");
	  break;
	case icIlluminantEquiPowerE  :
	  printf("EquiPower E\n");
	  break;
	case icIlluminantF8  :
	  printf("F8\n");
	  break;
	default :
	  printf("Unknown\n");
	  break;
	}
        printf("\n\n");	  
	break;


/**********************icSigMediaBlackPointTag********************************/


      case 0x626B7074L:  /* 'bkpt' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	if (fread(&xyz, sizeof(icXYZType), 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
        printf("XYZ type\n");
	printf("X=%f, Y=%f, Z=%f\n",
	       icfixed2double(xyz.data.data[0].X,
			      icSigS15Fixed16ArrayType),
	       icfixed2double(xyz.data.data[0].Y,
			      icSigS15Fixed16ArrayType),
	       icfixed2double(xyz.data.data[0].Z,
			      icSigS15Fixed16ArrayType));
        printf("\n\n");
	break;


/**********************icSigMediaWhitePointTag******************************/


      case 0x77747074L:  /* 'wtpt' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	if (fread(&xyz, sizeof(icXYZType), 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
        printf("XYZ type\n");
	printf("X=%f, Y=%f, Z=%f\n",
	       icfixed2double(xyz.data.data[0].X,
			      icSigS15Fixed16ArrayType),
	       icfixed2double(xyz.data.data[0].Y,
			      icSigS15Fixed16ArrayType),
	       icfixed2double(xyz.data.data[0].Z,
			      icSigS15Fixed16ArrayType));
        printf("\n\n");
	break;


/***********************icSigNamedColorTag******************************/


      case 0x6E636f6CL:  /* 'ncol'  OBSOLETE: use ncl2 */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
        printf("This is an obsolete tag - please use ncl2 \n\n");
	break;


/**********************icSigPreview0Tag******************************/


      case 0x70726530L:  /* 'pre0' */
	print_luttag(fd, tag[i], verbose, buf[i]);
	break;




/**********************icSigPreview1Tag*******************************/


      case 0x70726531L:  /* 'pre1' */
	print_luttag(fd, tag[i], verbose, buf[i]);
	break;



/**********************icSigPreview2Tag******************************/


      case 0x70726532L:  /* 'pre2' */
	print_luttag(fd, tag[i], verbose, buf[i]);
	break;



/**************************icSigProfileDescriptionTag**************************/


      case 0x64657363L:  /* 'desc' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	textdescalloc = (icTextDescriptionType *)calloc(tag[i].size, sizeof(icUInt8Number));
	if (fread(textdescalloc, tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	textdesc = &textdescalloc->desc;
        printf("Text description \n");
        printf("%s\n", textdesc->data);

        printf("\n\n");
	free(textdescalloc);
	break;


/**********************icSigProfileSequenceDescTag******************************/


      case 0x70736571L:  /* 'pseq' */

	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	profseqdescalloc = (icProfileSequenceDescType *)calloc(tag[i].size, sizeof(icUInt8Number));
	if (fread(profseqdescalloc,  tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	profseqdesc = &profseqdescalloc->desc;
	dataptr =  (char *)(icProfileSequenceDesc *)profseqdesc;
        profseqdesc->count = swap((long)profseqdesc->count);
	printf("Number of profile descriptions in tag = %d\n",profseqdesc->count );
	dataptr += sizeof(icUInt32Number);
	for (jj= 0;jj<profseqdesc->count; jj++) {
	  memcpy(&charstring, dataptr, sizeof(icSignature));

	  printf("device manufacturer  = %.4s\n", charstring);
	  dataptr +=  sizeof(icSignature);
	  memcpy(&charstring, dataptr, sizeof(icSignature));
	  printf("device model  = %.4s\n", charstring);

	  dataptr += sizeof(icSignature);
	  memcpy(&tmp64, dataptr,  sizeof(icUInt64Number));
	  printf("device attributes=  %d, %d\n",swap((long)tmp64[0]),swap((long)tmp64[1]));

	  dataptr += sizeof(icUInt64Number);
	  memcpy(&charstring, dataptr, sizeof(icSignature) );
	  printf("technology  = %.4s\n", charstring);

	  dataptr += sizeof(icSignature);

	  /* Device Manufacturer tag */
	  dataptr += 8;       /* Skip the tag base and save the count */
	  memcpy(&count, dataptr, sizeof(count));
          count = (icUInt32Number)swap(count);
	  dataptr += sizeof(count);
	  printf("Device manufacturer string length and contents = %d %s\n",count,  dataptr);

	/* 
	 * Skip over the string, UniCode count and type and
	 * ScriptCode stuff.
	 */
	  dataptr += count;   /* Ascii string */

	/* UniCode */
	  memcpy(&count, (dataptr + 4), sizeof(icUInt32Number));
	  /*count = swap((long)count);*/
	  dataptr += (2*sizeof(icUInt32Number) + count*2);
	
	/* ScriptCode */

	  dataptr += (sizeof(icUInt16Number) + sizeof(icUInt8Number)
                        + 67);
	
	/* Device Model Tag */
	  dataptr += 8;       /* Skip the tag base and count */
	  memcpy(&count, dataptr, sizeof(icUInt32Number));
	  count = swap((long)count);
	  dataptr += sizeof(count);
	  printf("Device model string length and contents = %d  %s\n",count, dataptr);
	
	/* 
	 * Skip over the string, UniCode count and type and
	 * ScriptCode stuff.
	 */
	  dataptr += count;   /* Ascii string */

	/* UniCode */
	  memcpy(&count, (dataptr + 4), sizeof(icUInt32Number));
	  count = swap((long)count);
	  dataptr += (2*sizeof(icUInt32Number) + count*2);
	
	/* ScriptCode */
	  dataptr += (sizeof(icUInt16Number) + sizeof(icUInt8Number)
                        + 67);
	
	  printf("\n\n");
	}
	free(profseqdescalloc);
	break;


/*********************icSigPs2CRD0Tag*******************************/


      case 0x70736430L:  /* 'psd0' */

	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);

	datatalloc = (icDataType *)calloc(tag[i].size, sizeof(icUInt8Number));
	if (fread(datatalloc,tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	datat = &datatalloc->data;
        printf("Data type\n");
        if (swap((long)datat->dataFlag) == 0) {
            printf("Ascii data \n");
            printf("%s", datat->data);
        } else {
            printf("Binary data \n");
            printf("Size = %d bytes\n",(tag[j].size - 8 -
					sizeof(datat->dataFlag) ));
        }

        printf("\n\n");
	free(datatalloc);
	break;


/*********************icSigPs2CRD1Tag*******************************/


      case 0x70736431L:  /* 'psd1' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);

	datatalloc = (icDataType *)calloc(tag[i].size, sizeof(icUInt8Number));
	if (fread(datatalloc,tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	datat = &datatalloc->data;
        printf("Data type\n");
        if (swap((long)datat->dataFlag) == 0) {
            printf("Ascii data \n");
            printf("%s", datat->data);
        } else {
            printf("Binary data \n");
            printf("Size = %d bytes\n",(tag[j].size - 8 -
					sizeof(datat->dataFlag) ));
        }

        printf("\n\n");
	free(datatalloc);
	break;


/*********************icSigPs2CRD2Tag*******************************/


      case 0x70736432L:  /* 'psd2' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);

	datatalloc = (icDataType *)calloc(tag[i].size, sizeof(icUInt8Number));
	if (fread(datatalloc,tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	datat = &datatalloc->data;
        printf("Data type\n");
        if (swap((long)datat->dataFlag) == 0) {
            printf("Ascii data \n");
            printf("%s", datat->data);
        } else {
            printf("Binary data \n");
            printf("Size = %d bytes\n",(tag[j].size - 8 -
					sizeof(datat->dataFlag) ));
        }

        printf("\n\n");
	free(datatalloc);
	break;


/********************icSigPs2CRD3Tag*********************************/


      case 0x70736433L:  /* 'psd3' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);

	datatalloc = (icDataType *)calloc(tag[i].size, sizeof(icUInt8Number));
	if (fread(datatalloc,tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	datat = &datatalloc->data;
        printf("Data type\n");
        if (swap((long)datat->dataFlag) == 0) {
            printf("Ascii data \n");
            printf("%s", datat->data);
        } else {
            printf("Binary data \n");
            printf("Size = %d bytes\n",(tag[j].size - 8 -
					sizeof(datat->dataFlag) ));
        }

        printf("\n\n");
	free(datatalloc);
	break;


/********************icSigPs2CSATag********************************/


      case 0x70733273L:  /* 'ps2s' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);

	datatalloc = (icDataType *)calloc(tag[i].size, sizeof(icUInt8Number));
	if (fread(datatalloc,tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	datat = &datatalloc->data;
        printf("Data type\n");
        if (swap((long)datat->dataFlag) == 0) {
            printf("Ascii data \n");
            printf("%s", datat->data);
        } else {
            printf("Binary data \n");
            printf("Size = %d bytes\n",(tag[j].size - 8 -
					sizeof(datat->dataFlag) ));
        }

        printf("\n\n");
	free(datatalloc);

	break;


/***********************icSigPs2RenderingIntentTag******************************/


      case 0x70733269L:  /* 'ps2i' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);

	datatalloc = (icDataType *)calloc(tag[i].size, sizeof(icUInt8Number));
	if (fread(datatalloc,tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	datat = &datatalloc->data;
        printf("Data type\n");
        if (swap((long)datat->dataFlag) == 0) {
            printf("Ascii data \n");
            printf("%s", datat->data);
        } else {
            printf("Binary data \n");
            printf("Size = %d bytes\n",(tag[j].size - 8 -
					sizeof(datat->dataFlag) ));
        }

        printf("\n\n");
	free(datatalloc);
	break;


/**********************icSigRedColorantTag******************************/


      case 0x7258595AL:  /* 'rXYZ' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	if (fread(&xyz, sizeof(icXYZType), 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
        printf("XYZ type\n");
	printf("X=%f, Y=%f, Z=%f\n",
	       icfixed2double(xyz.data.data[0].X,
			      icSigS15Fixed16ArrayType),
	       icfixed2double(xyz.data.data[0].Y,
			      icSigS15Fixed16ArrayType),
	       icfixed2double(xyz.data.data[0].Z,
			      icSigS15Fixed16ArrayType));
        printf("\n\n");
	break;


/********************icSigRedTRCTag********************************/


      case 0x72545243L:  /* 'rTRC' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	curvealloc = (icCurveType *)calloc(tag[i].size, sizeof(icUInt8Number));
	if (fread(curvealloc,  tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	curve = &curvealloc->curve;
	curve->count = swap((long)curve->count);
        printf("Curve type, curve count = %d\n",curve->count );
            /* Linear */
            if (curve->count == 0) {
                printf("Count = 0 Curve is linear\n");
            }
            /* Gamma value */
            else if (curve->count == 1) {
                printf("Count = 1 Curve is a gamma of %f\n",
                       icfixed2gamma((icUInt16Number)curve->data[0]));
            }
            /* Beginning/end points of a line */
            else if (curve->count == 2) {
                printf("Count = 2 Line Start = %d End = %d\n",
                      (icUInt16Number)swap16((ushort)curve->data[0]), 
		       swap16((ushort)(icUInt16Number)curve->data[1])); 
            } else {

	    /* Just a plain old curve */
	    for (j=0; j<curve->count; j++) {
	      printf("%d ", (icUInt16Number)swap16((ushort)curve->data[j]));
	      if (!(j%16) && (j!=0))
                printf("\n");
	    }
	    }
        printf("\n\n");
	free(curvealloc);

	break;


/*********************icSigScreeningDescTag*******************************/


      case 0x73637264L:  /* 'scrd' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	textdescalloc = (icTextDescriptionType *)calloc(tag[i].size, sizeof(icUInt8Number));
	if (fread(textdescalloc, tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	textdesc = &textdescalloc->desc;
        printf("Text description \n");
        printf("%s\n", textdesc->data);

        printf("\n\n");
	free(textdescalloc);
	break;


/**********************icSigScreeningTag******************************/


      case 0x7363726EL:  /* 'scrn' */

	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	screenalloc = (icScreeningType *)calloc(tag[i].size, sizeof(icUInt8Number));
	if (fread(screenalloc, tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	screen = &screenalloc->screen;
        printf("Screening data\n");
        printf("Number of channels = %d\n", swap((long)screen->channels));
        printf("Screening flag = %d\n", swap((long)screen->screeningFlag));
        scptr = screen->data;
        for (j=0; j<swap((long)screen->channels); j++) {
            printf("Channel %d\n", j);
            printf("Frequency = %f\n",
                   icfixed2double(scptr->frequency, icSigS15Fixed16ArrayType));
            printf("Angle = %f\n",
                   icfixed2double(scptr->angle, icSigS15Fixed16ArrayType));
            printf("SpotShape = %d\n", swap((long)scptr->spotShape));
            scptr++;
        }

        printf("\n\n");
	free(screenalloc);
	break;


/**********************icSigTechnologyTag******************************/


      case 0x74656368L:  /* 'tech' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	if (fread(&tech, sizeof(icSignatureType), 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	switch(swap((long)tech.signature)) {

  	case 0x6463616DL:  /* 'dcam' */
	  printf("technology type is DigitalCamera \n");
	  break;
  	case 0x6673636EL:  /* 'fscn' */
	  printf("technology type is FilmScanner \n");
	  break;
 	case 0x7273636EL:  /* 'rscn' */
	  printf("technology type is ReflectiveScanner  \n");
	  break;
 	case 0x696A6574L:  /* 'ijet' */ 
	  printf("technology type is InkJetPrinter \n"); 
	  break;
 	case 0x74776178L:  /* 'twax' */
	  printf("technology type is ThermalWaxPrinter \n"); 
	  break;
 	case 0x6570686FL:  /* 'epho' */
	  printf("technology type is ElectrophotographicPrinter \n"); 
	  break;
	case 0x65737461L:  /* 'esta' */
	  printf("technology type is ElectrostaticPrinter \n");           
	  break;
	case 0x64737562L:  /* 'dsub' */
	  printf("technology type is DyeSublimationPrinter \n");          
	  break;
	case 0x7270686FL:  /* 'rpho' */
	  printf("technology type is PhotographicPaperPrinter \n");       
	  break;
	case 0x6670726EL:  /* 'fprn' */
	  printf("technology type is FilmWriter \n");                     
	  break;
	case 0x7669646DL:  /* 'vidm' */
	  printf("technology type is VideoMonitor \n");                   
	  break;
	case 0x76696463L:  /* 'vidc' */
	  printf("technology type is VideoCamera \n");                    
	  break;
 	case 0x706A7476L:  /* 'pjtv' */
	  printf("technology type is ProjectionTelevision \n");          
	  break;
   	case 0x43525420L:  /* 'CRT ' */
	  printf("technology type is CRTDisplay \n");                  
	  break;
  	case 0x504D4420L:  /* 'PMD ' */
	  printf("technology type is PMDisplay \n");                    
	  break;
  	case 0x414D4420L:  /* 'AMD ' */
	  printf("technology type is AMDisplay \n");                    
	  break;
  	case 0x4B504344L:  /* 'KPCD' */
	  printf("technology type is PhotoCD \n");                      
	  break;
  	case 0x696D6773L:  /* 'imgs' */
	  printf("technology type is PhotoImageSetter \n");             
	  break;
 	case 0x67726176L:  /* 'grav' */
	  printf("technology type is Gravure \n");                       
	  break;
	case 0x6F666673L:  /* 'offs' */
	  printf("technology type is OffsetLithography \n");              
	  break;
 	case 0x73696C6BL:  /* 'silk' */
	  printf("technology type is Silkscreen \n");                    
	  break;
 	case 0x666C6578L:  /* 'flex' */
	  printf("technology type is Flexography \n");                   
	  break;	}
        printf("\n\n");
	break;


/********************icSigUcrBgTag********************************/


      case 0x62666420L:  /* 'bfd ' */
	printf("signature 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	ucrbgalloc = (icUcrBgType *)calloc(tag[i].size, sizeof(icUInt8Number));
	if (fread(ucrbgalloc, tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	ucrbg = &ucrbgalloc->data;
	dataptr = (char *)ucrbg->data;

        printf("UCR BG description\n");

        /* Do everything by bytes */

        /* UCR Curve */
        memcpy(&count, dataptr, 4);
	count = swap((long)count);
        printf("UCR curve length = %d\n", count);
        dataptr += 4;
        if (count == 1) {
            /* Its a percentage */
            memcpy(&val, dataptr, 2);
            printf("UCR = %d percent\n", swap16((ushort)val));
            dataptr += 2;
        } else {
            for (j=0; j<count; j++) {
                memcpy(&val, dataptr, 2);
                printf("%d\n",swap16((ushort)val));
                dataptr += 2;
            }
        }

        /* BG Curve */
        memcpy(&count, dataptr, 4);
	count = swap((long)count);
        printf("BG curve length = %d\n", count);
        dataptr += 4;
        if (count == 1) {
            /* Its a percentage */
            memcpy(&val, dataptr, 2);
            printf("BG = %d percent\n", swap16((ushort)val));
            dataptr += 2;
        } else {
            for (j=0; j<count; j++) {
                memcpy(&val, dataptr, 2);
                printf("%d\n", swap16((ushort)val));
                dataptr += 2;
            }
        }

        printf("UcrBg description = %s\n", dataptr);
	free(ucrbgalloc);

        printf("\n\n");
	break;


/************************icSigViewingCondDescTag****************************/


      case 0x76756564L:  /* 'vued' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	textdescalloc = (icTextDescriptionType *)calloc(tag[i].size, sizeof(icUInt8Number));
	if (fread(textdescalloc, tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	textdesc = &textdescalloc->desc;
        printf("Text description \n");
        printf("%s\n", textdesc->data);

        printf("\n\n");
	free(textdescalloc);
	break;


/**************************icSigViewingConditionsTag**************************/


       case 0x76696577L:  /* 'view' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	if (fread(&view, tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
        printf("Viewing conditions type\n");
            printf("Illuminant X=%f, Y=%f, Z=%f\n",
                   icfixed2double(view.view.illuminant.X,
                                  icSigS15Fixed16ArrayType),
                   icfixed2double(view.view.illuminant.Y,
                                  icSigS15Fixed16ArrayType),
                   icfixed2double(view.view.illuminant.Z,
                                  icSigS15Fixed16ArrayType));
            printf("Surround X=%f, Y=%f, Z=%f\n",
                   icfixed2double(view.view.surround.X,
                                  icSigS15Fixed16ArrayType),
                   icfixed2double(view.view.surround.Y,
                                  icSigS15Fixed16ArrayType),
                   icfixed2double(view.view.surround.Z,
                                  icSigS15Fixed16ArrayType));
            printf("StdIlluminant %d\n", 
                   swap((long)view.view.stdIluminant));

        printf("\n\n");
	break;


/*********************icSigNamedColor2Tag*******************************/


      case 0x6E636C32L:  /* 'ncl2' */
	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	nc2alloc = (icNamedColor2Type *)calloc(tag[i].size, sizeof(icUInt8Number));
	if (fread(nc2alloc, tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	nc2 = &nc2alloc->ncolor;
	dataptr = (char *) (icNamedColor2 *)nc2->data;
        printf("Named color type\n");
        printf("Vendor = %d\n",swap((long)nc2->vendorFlag));
        printf("Count = %d\n", swap((long)nc2->count));
        printf("Number device coordinates = %d\n", swap((long)nc2->nDeviceCoords));
        printf("Color prefix = %s\n", nc2->prefix);
        printf("Color suffix = %s\n",nc2->suffix);
        jj = 0;

        for (ii=0; ii<swap((long)nc2->count); ii++) {
            printf("\nColor  %d  Root name =  ", ii+1);
	    printf("%s",dataptr);
            dataptr +=32;
            jj +=32;
            printf("\nPCS Coordinates=  ");

            for (j=0; j< 3; j++) {
                memcpy(&val, dataptr, sizeof(icUInt16Number));
                printf("%d ", swap16((ushort)val));
                dataptr += sizeof(icUInt16Number);
                jj += sizeof(icUInt16Number);
                if (!(j%16) && (j!=0)) printf("\n");
            }
            printf("\nDevice Coordinates=  ");
       
            for (j=0; j<swap((long)nc2->nDeviceCoords); j++) {
                memcpy(&val, dataptr, sizeof(icUInt16Number));
                printf("%d ",  swap16((ushort)val));
                dataptr += sizeof(icUInt16Number);
                jj += sizeof(icUInt16Number);
                if (!(j%16) && (j!=0)) printf("\n");
            }
            printf("\n");
        }
	free(nc2alloc);
        printf("\n\n");
	break;


/**********************icSigCrdInfoTag*******************************/


      case 0x63726469L:  /* 'crdi' */

	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	crdalloc = (icCrdInfoType *)calloc(tag[i].size, sizeof(icUInt8Number));
	if (fread(crdalloc, tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	crd = &crdalloc->info;
	dataptr = crd->data;
        printf("CRD Info type\n");
        crd->count = swap((long)crd->count);
	printf("PostScript Product name count and string =  %d,  %s\n", crd->count, dataptr);
	dataptr += crd->count;
	for (ii=0; ii<4; ii++) {
          printf("\n");
	  memcpy(&count, dataptr, sizeof(icUInt32Number));
          count = swap((long)count);
	  dataptr+= sizeof(icUInt32Number);
	  printf("Rendering Intent %d CRD count and name =  %d, %s\n", ii, count, dataptr);
	  dataptr += count;
		       }
        printf("\n\n");
	free(crdalloc);
	break;


/**********************icSigDeviceSettingsTag*******************************/


      case 0x64657673L:  /* 'devs' */

	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	settingstype = (icDeviceSettingsType *)calloc(tag[i].size, sizeof(icUInt8Number));
	settingsdata = &settingstype->data;

	if (fread(settingstype, tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	ptr32 = 0;
        printf("Device Settings type\n");
        settingsdata->numPlatforms = swap((long)settingsdata->numPlatforms);
	printf("Number of Platforms %d \n", settingsdata->numPlatforms);
	for (ii=0; ii< settingsdata->numPlatforms; ii++) {
	  switch( swap((long)settingsdata->data[ptr32++]) ) {
	  case icSigMacintosh:
	    printf("platform = Macintosh\n");
	    break;
	  case icSigMicrosoft:
	    printf("platform = Microsoft\n");
	    break;
	  case icSigSolaris:
	    printf("platform = Solaris\n");
	    break;
	  case icSigSGI:
	    printf("platform = SGI\n");
	    break;
	  case icSigTaligent:
	    printf("platform = Taligent\n");
	    break;
	  default:
	    printf("Unknown\n");
	    break;
	  }
	  printf("Size of this platform structure %d \n", swap((long)settingsdata->data[ptr32++]));
	  ii2 = swap((long)settingsdata->data[ptr32++]);
	  printf("Number of combinations %d \n", ii2);
	  for (j=0; j<ii2; j++) {
	    printf("Size of this combination structure %d \n",  swap((long)settingsdata->data[ptr32++]));
	    jj2 = swap((long)settingsdata->data[ptr32++]);
	    printf("Number of structures %d \n", jj2);
	    for (jj=0; jj<jj2; jj++) {
	      printf("Settings signature =");
	      switch (swap((long)settingsdata->data[ptr32++])) {
	      case icSigResolution:
		printf(" Resolution\n");
		printf("Size of of setting value = %d\n",swap((long)settingsdata->data[ptr32++]));
		ii3 = swap((long)settingsdata->data[ptr32++]);
		printf("Number of resolution settings = %d\n", ii3);
		for (ii4=0; ii4<ii3; ii4++){
#ifdef _LITTLE_ENDIAN
		  /*swap the printing of the 2 32 bit words and then each word is swapped*/
		  printf("DPI,  X and Y resolution = %d %d \n", 
		  swap((long)settingsdata->data[ptr32++]), swap((long)settingsdata->data[ptr32++]));
#else
		  printf("DPI,  Y and X resolution = %d %d \n", 
		  settingsdata->data[ptr32++], settingsdata->data[ptr32++]);
#endif
		}
		break;
	      case icSigMedia:
		printf(" Media\n");
		printf("Size of of setting value = %d\n",swap((long)settingsdata->data[ptr32++]));
		ii3 = swap((long)settingsdata->data[ptr32++]);
		printf("Number of media settings = %d\n", ii3);
		for (ii4=0; ii4<ii3; ii4++){
		printf("Device Media = ");
		switch(swap((long)settingsdata->data[ptr32++])) {
		case icStandard:
		  printf("Standard\n");
		  break;
		case icTrans:
		  printf("Transparency\n");
		  break;
		case icGloss:
		  printf("Glossy\n");
		  break;
		case icUser1:
		  printf("User defined\n");
		  break;
		default:
		  printf(" Unknown\n");
		  break;
		}
		}
		break;
	      case icSigHalftone:
		printf(" Halftone\n");
		printf("Size of of setting value = %d\n",swap((long)settingsdata->data[ptr32++]));
		ii3 = swap((long)settingsdata->data[ptr32++]);
		printf("Number of halftone settings = %d\n", ii3);
		for (ii4=0; ii4<ii3; ii4++){
		switch(swap((long)settingsdata->data[ptr32++])) {
		case icNone:
		  printf("None\n");
		  break;
		case icCoarse:
		  printf("Coarse\n");
		  break;
		case icFine:
		  printf("Fine\n");
		  break;
		case icLineArt:
		  printf("Line art\n");
		  break;
		case icErrorDiffusion:
		  printf("Error diffusion\n");
		  break;
		case icReserved6:
		  printf("Reserved \n");
		  break;
		case icReserved7:
		  printf("Reserved \n");
		  break;
		case icReserved8:
		  printf("Reserved \n");
		  break;
		case icReserved9:
		  printf("Reserved \n");
		  break;
		case icGrayScale:
		  printf("Gray scale \n");
		  break;
		case icUser2:
		  printf("User defined \n");
		  break;
		default:
		  printf(" Unknown\n");
		  break;
		}
		}
		break;
	      default:
		printf(" Unknown\n");
		break;
	      }
	    }
	  }
	}
        printf("\n\n");
	free(settingstype);
	break;


/**********************icResponseCurveSet16 Tag*******************************/


      case 0x72657370L:  /* 'resp' */

	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	responsetype = (icResponseCurveSet16Type *)calloc(tag[i].size, sizeof(icUInt8Number));
	responsedata = &responsetype->data;

	if (fread(responsetype, tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	ptr32 = 0;
        printf("Response Curve type\n");
        responsedata->channels = swap16((ushort)responsedata->channels);
        responsedata->numTypes = swap16((ushort)responsedata->numTypes);
	printf("Number of Channels %d \n", responsedata->channels);
	printf("Number of response types %d \n",  responsedata->numTypes);
	for (ii=0; ii< responsedata->numTypes; ii++) {
	  printf("Byte offset to type %d = %d \n", ii, swap((long)responsedata->data[ptr32++]) );
	}
	for (ii=0; ii< responsedata->numTypes; ii++) {
	  printf("Type %d = ", ii+1);	
	  switch( swap((long)responsedata->data[ptr32++]) ) {
	  case icStaA:
	    printf("StaA \n");
	    break;
	  case icStaE:
	    printf("StaE \n");
	    break;
	  case icStaI:
	    printf(" StaI\n");
	    break;
	  case icStaT:
	    printf("StaT \n");
	    break;
	  case icStaM:
	    printf("StaM \n");
	    break;
	  case icDN:
	    printf("DN \n");
	    break;
	  case icDNP:
	    printf("DNP \n");
	    break;
	  case icDNN:
	    printf("DNN \n");
	    break;
	  case icDNNP:
	    printf("DNNP \n");
	    break;
	  default:
	    printf("Unknown\n");
	    break;
	  }
	  ii2 = 0;
	  for (jj=0; jj<responsedata->channels; jj++) {
	    channelvals[ii2] = swap((long)responsedata->data[ptr32++]);
	    printf("# measurements for channel %d = %d\n",jj+1,channelvals[ii2]);
	    ii2++;
	  }
	  for (jj=0; jj<responsedata->channels; jj++) {
	      printf(" %f  ",icfixed2double(responsedata->data[ptr32++], icSigS15Fixed16ArrayType));
	      printf(" %f  ",icfixed2double(responsedata->data[ptr32++], icSigS15Fixed16ArrayType));
	      printf(" %f  \n",icfixed2double(responsedata->data[ptr32++], icSigS15Fixed16ArrayType));
	}
	  for (jj=0; jj<responsedata->channels; jj++) {
	    printf("response data for channel %d\n", jj+1);
	    for (jj2=0; jj2<channelvals[jj]; jj2++) {
	      printf("Interval = %d ", (swap((long)responsedata->data[ptr32++])>>16));
	      printf("value %f  \n",icfixed2double(responsedata->data[ptr32++], 
						 icSigS15Fixed16ArrayType));
	    }
	    printf("\n");	 
          }
          printf("\n");	    
	  }
        printf("\n\n");
	free(responsetype);
	break;

      default:
	printf("private tag:  signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);

        printf("\n\n");
	break;

/********************icSigCromaticityTag********************************/


      case 0x6368726dL:  /* 'chrm ' */
	printf("signature 0x%x signatureId = %s, offset = %d size = %d\n",
		tag[i].sig, buf[i], tag[i].offset, tag[i].size);
	fseek(fd, (long) tag[i].offset, 0);
	chromaalloc = (icChromaticityType *)calloc(tag[i].size, 
				      sizeof(icUInt8Number));
	if (fread(chromaalloc, tag[i].size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	chroma = &chromaalloc->chromaticity;

        printf("Chromaticity description\n");
	chroma->channels = swap16((ushort)chroma->channels);
	printf("Number of channels %d\n", chroma->channels);
	printf("Phopshor or Colorant type = ");
	switch( (int)swap16((ushort)chroma->type) ) {
	  case icITURBT709:
	    printf("ITU-R BT.709 \n");
	    break;
	  case icSMPTERP1451994:
	    printf("SMPTE RP145-1994\n");
	    break;
	  case icEBUTech3213E:
	    printf("EBU Tech.3213-E\n");
	    break;
	  case icP22:
	    printf("P22\n");
	    break;
	  default:
	    printf("Unknown\n");
	    break;
	  }
        /* Do everything by bytes */
	ii =0;
	for (j=0; j<chroma->channels; j++) {
	  /*	  memcpy(n32, chroma->data[j], 2);*/

	  printf(" %f  ",icfixed2double(chroma->data[ii++], 
			 icSigU16Fixed16ArrayType));
	  /*	  dataptr += 2;
	  memcpy(&val, chroma->data[j+1], 2);*/
	  printf(" %f  ",icfixed2double(chroma->data[ii++], 
			 icSigU16Fixed16ArrayType));
	  /*	  dataptr += 2;*/
	}

	free(chromaalloc);

        printf("\n\n");
	break;

	}
      }
     exit(0);

/*******************Examples for writing tags*******************/


    } else {  
      /*For example purposes, the data for each tag is hard-coded in each tag case.
	One would normally fill in the tags via an input data file or an interactive
	application, etc.  This example will create a profile with an example of 
	every tag - not a typical or even valid scenerio - but it works for 
	instruction.  icc.h structures will be used.*/

      /*First, indicate the number of tags to be entered in the profile, not counting
	the header.  For this example, we do not include all tags possible.  The
	size of the header and tag directory can be calculated.  Header = 128 bytes
        and each tag index in the tag directory is 12 bytes. */


      for (i=0; i<4; i++) tagbase.reserved[i] = 0;
      /* total ICC defined ntags = 42 - not all examples are written;*/
      ntags = 26;
      size = 128 + 4 + (12 *ntags);		/*header + ntags + tagdir */
      dirptr = 0;
      tagdataptr = size;				/*point to start of tag data*/


/******************header**********************************/

      hdr = (icHeader *)calloc(sizeof(icHeader), sizeof(icUInt8Number));
      hdr->size = swap((size + 4)); /*update with final profile size*/
      hdr->cmmId = swap((long)0x4b434d53L);		/*'KCMS'*/
      hdr->version = swap((long)0x21000000L);
      hdr->deviceClass = swap((long)0x6d6e7472L);	/*'mntr'*/
      hdr->colorSpace = swap((long)0x52474220L);	/*'RGB '*/
      hdr->pcs = swap((long)0x4c616220L);		/*'Lab '*/
      hdr->cmmId = swap((long)0x4b434d53L);  		/*'KCMS'*/
      hdr->version= swap((long)0x21000000L);
      hdr->deviceClass = swap((long)0x6d6e7472L);	/*'mntr'*/
      hdr->colorSpace = swap((long)0x52474220L);	/*'RGB '*/
      hdr->pcs = swap((long)0x4c616220L);		/*'Lab '*/
      /* Get the time from the system */
      clocktime = time(NULL);
      tmdatetime = localtime(&clocktime);

      hdr->date.seconds = (icUInt16Number)swap16((ushort)tmdatetime->tm_sec);
      hdr->date.minutes = (icUInt16Number)swap16((ushort)tmdatetime->tm_min);
      hdr->date.hours = (icUInt16Number)swap16((ushort)tmdatetime->tm_hour);
      hdr->date.day = (icUInt16Number)swap16((ushort)tmdatetime->tm_mday);
      hdr->date.month = (icUInt16Number)swap16((ushort)(tmdatetime->tm_mon +1));
      hdr->date.year = (icUInt16Number)swap16((ushort)(tmdatetime->tm_year + 1900));

      hdr->magic = swap((long)0x61637370L);		/*'acsp'*/
      hdr->platform = swap((long)0x53554e57L);		/*'SUNW'*/
      hdr->flags = swap((long)0x00000000L);		/*Profile not embedded and
							can be used independently*/
      /*manufacturer and model should be registered with the ICC*/
      hdr->manufacturer = swap((long)0x53554e20L);	/*'SUN '*/
      hdr->model = swap((long)0x31393938L);		/*'1998'*/
      hdr->attributes[0] = swap((long)9);     /*Transparency, glossy, positive, B&W*/
      hdr->attributes[1] = swap((long)0); 	
      hdr->renderingIntent = swap((long)1 << 16);      /*relative colorimetric*/
      hdr->illuminant.X = double2icfixed((double)0.964188, icSigS15Fixed16ArrayType);
      hdr->illuminant.Y = double2icfixed((double)1.000, icSigS15Fixed16ArrayType);
      hdr->illuminant.Z = double2icfixed((double)0.824890, icSigS15Fixed16ArrayType);
      hdr->creator = swap((long)0x53554e57L);		/*'SUNW'*/
      if (fwrite(hdr, sizeof(icHeader), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}
      fseek(fd, sizeof(icHeader), SEEK_SET);
      ntags = swap((long)ntags);
      if (fwrite(&ntags, sizeof(icUInt32Number), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}
      dirptr =  sizeof(icHeader) + 4;	       /*point to start of tag directory*/
      free(hdr);


/*********************A2B0 tag********************************/


      tagdir.sig = swap((long)0x41324230L);	/*'A2B0'*/
      tagdir.offset = swap((long)tagdataptr);
      tblenum = 6;				/*input/ouptut tables will 
						  contain 3X2 values each*/
      lutnum = 1536;				/*lut will be 8X8X8 X3chanels */
      size = 52 + 				/*tag base + pre-table data*/
	(4 * tblenum) + (2*lutnum) + (4*tblenum);
      printf("tag size = %d\n",size);
      tagdir.size = swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	} 
      dirptr += sizeof(tagdir);			/*keep this pointed to
						the end of the tag directory*/
      /*create a null lut 16 tag for testing purposes*/
      tagbase.sig = swap((long)0x6d667432L);	/*'mft2'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}   
      tagdataptr += sizeof(tagbase);
      lut16alloc = (icLut16Type *)calloc(size, sizeof(icUInt8Number));
      lut16 = &lut16alloc->lut;
      lut16->inputChan = (icUInt8Number)3;
      lut16->outputChan = (icUInt8Number)3;
      lut16->clutPoints = (icUInt8Number)8;	 /*per-side count*/
      lut16->pad = (icInt8Number)0;

      /*create an identity matrix*/
      lut16->e00 =  double2icfixed(1.000, icSigS15Fixed16ArrayType);
      lut16->e01 =  double2icfixed(0.000, icSigS15Fixed16ArrayType);
      lut16->e02 =  double2icfixed(0.000, icSigS15Fixed16ArrayType);
      lut16->e10 =  double2icfixed(0.000, icSigS15Fixed16ArrayType);
      lut16->e11 =  double2icfixed(1.000, icSigS15Fixed16ArrayType);
      lut16->e12 =  double2icfixed(0.000, icSigS15Fixed16ArrayType);
      lut16->e20 =  double2icfixed(0.000, icSigS15Fixed16ArrayType);
      lut16->e21 =  double2icfixed(0.000, icSigS15Fixed16ArrayType);
      lut16->e22 =  double2icfixed(1.000, icSigS15Fixed16ArrayType);

      lut16->inputEnt = (icUInt16Number)swap16((ushort)2);/*will be 2X#chan total entries*/
      lut16->outputEnt = (icUInt16Number)swap16((ushort)2);
      /*input tables -  values supplied in the uInt16Number range  */
      i = 0;
      lut16->data[i++] = (icUInt16Number)swap16((ushort)0);
      lut16->data[i++] = (icUInt16Number)swap16((ushort)65535);
      lut16->data[i++] = (icUInt16Number)swap16((ushort)0);
      lut16->data[i++] = (icUInt16Number)swap16((ushort)65535);
      lut16->data[i++] = (icUInt16Number)swap16((ushort)0);
      lut16->data[i++] = (icUInt16Number)swap16((ushort)65535);
      /*clut table*/
      for (ri = 0; ri< 256; ri+=32)
	{
	  for (gi = 0; gi < 256; gi+=32)
	    {
	      for (bi = 0; bi < 256; bi+=32)
		{
		  lut16->data[i++] = (icUInt16Number)swap16((ushort)(min (max (0, (int)
								(ri * 255)), 65535)));
		  lut16->data[i++] = (icUInt16Number)swap16((ushort)(min (max (0, (int)
								(gi * 255)), 65535)));
		  lut16->data[i++] = (icUInt16Number)swap16((ushort)(min (max (0, (int)
								(bi * 255)), 65535)));
		} 
	    } 
	} 
      /*output tables */
      lut16->data[i++] = (icUInt16Number)swap16((ushort)0);
      lut16->data[i++] = (icUInt16Number)swap16((ushort)65535);
      lut16->data[i++] = (icUInt16Number)swap16((ushort)0);
      lut16->data[i++] = (icUInt16Number)swap16((ushort)65535);
      lut16->data[i++] = (icUInt16Number)swap16((ushort)0);
      lut16->data[i++] = (icUInt16Number)swap16((ushort)65535);
      if (fwrite(lut16, size - sizeof(tagbase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}
      tagdataptr += size - sizeof(tagbase);
      /*update the final size of profile in the header*/
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {	
	  printf("error writing file\n");
	  exit(1);
	}
      free(lut16alloc);
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;



/*********************A2B1 tag********************************/


      tagdir.sig = swap((long)0x41324231L);	/*'A2B1'*/
      tagdir.offset = swap((long)tagdataptr);
      tblenum = 3 * 256;			/*input/ouptut tables will 
						  contain 256 values each*/
      lutnum = 1536;				/*lut will be 8X8X8 X3chanels */
      size = 48 + 				/*tag base + pre-table data*/
	(tblenum) + (lutnum) + (tblenum); 	 /*byte data*/
      printf("tag size = %d\n",size);
      tagdir.size = swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	} 
      dirptr += sizeof(tagdir);			/*keep this pointed to the end of the 
						  tag directory*/
      /*create a null lut 8 tag for testing purposes*/
      tagbase.sig = swap((long)0x6d667431L);	/*'mft1'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}   
      tagdataptr += sizeof(tagbase);
      lut8alloc = (icLut8Type *)calloc(size, sizeof(icUInt8Number));
      lut8 = &lut8alloc->lut;
      lut8->inputChan = (icUInt8Number)3;
      lut8->outputChan = (icUInt8Number)3;
      lut8->clutPoints = (icUInt8Number)8;	 /*per-side count*/
      lut8->pad = (icInt8Number)0;

      /*create an identity matrix*/
      lut8->e00 =  double2icfixed(1.000, icSigS15Fixed16ArrayType);
      lut8->e01 =  double2icfixed(0.000, icSigS15Fixed16ArrayType);
      lut8->e02 =  double2icfixed(0.000, icSigS15Fixed16ArrayType);
      lut8->e10 =  double2icfixed(0.000, icSigS15Fixed16ArrayType);
      lut8->e11 =  double2icfixed(1.000, icSigS15Fixed16ArrayType);
      lut8->e12 =  double2icfixed(0.000, icSigS15Fixed16ArrayType);
      lut8->e20 =  double2icfixed(0.000, icSigS15Fixed16ArrayType);
      lut8->e21 =  double2icfixed(0.000, icSigS15Fixed16ArrayType);
      lut8->e22 =  double2icfixed(1.000, icSigS15Fixed16ArrayType);

      /*input tables - each must be 256 values - supplied in the UInt8Number range */
      i = 0;
      for (j=0; j<256; j++) {
	lut8->data[i++] = (icUInt8Number)j;
      }
      for (j=0; j<256; j++) {
	lut8->data[i++] = (icUInt8Number)j;
      }
      for (j=0; j<256; j++) {
	lut8->data[i++] = (icUInt8Number)j;
      }
      /*clut*/
      for (ri = 0; ri< 256; ri+=32)
	{
	  for (gi = 0; gi < 256; gi+=32)
	    {
	      for (bi = 0; bi < 256; bi+=32)
		{
		  lut8->data[i++] = (icUInt8Number)(min (max (0, (int)
								(ri * 255)), 65535));
		  lut8->data[i++] = (icUInt8Number)(min (max (0, (int)
								(gi * 255)), 65535));
		  lut8->data[i++] = (icUInt8Number)(min (max (0, (int)
								(bi * 255)), 65535));
		} 
	    } 
	} 
      /*output tables - each must be 256 values*/
      for (j=0; j<256; j++) {
	lut8->data[i++] = (icUInt8Number)j;
      }
      for (j=0; j<256; j++) {
	lut8->data[i++] = (icUInt8Number)j;
      }
      for (j=0; j<256; j++) {
	lut8->data[i++] = (icUInt8Number)j;
      }

      if (fwrite(lut8, size - sizeof(tagbase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}
      tagdataptr += size - sizeof(tagbase);
      /*update the final size of profile in the header*/
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {	
	  printf("error writing file\n");
	  exit(1);
	}
      free(lut8alloc);
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;



/**********************bXYZ tag******************************/


      tagdir.sig = swap((long)0x6258595aL);			/*'bXYZ'*/
      tagdir.offset = swap((long)tagdataptr);
      size = sizeof(tagbase) + sizeof(icXYZNumber);
      printf("tag size = %d\n",size);
      tagdir.size = swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	} 



      dirptr += sizeof(tagdir);			/*keep this pointed to the end of the 
						  tag directory*/
      tagbase.sig = swap((long)0x58595a20L);		/*'XYZ '*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}   
      tagdataptr +=sizeof(tagbase);
      xyz.data.data[0].X = double2icfixed (0.279984, icSigS15Fixed16ArrayType);
      xyz.data.data[0].Y = double2icfixed (0.200272, icSigS15Fixed16ArrayType);
      xyz.data.data[0].Z = double2icfixed (0.840454, icSigS15Fixed16ArrayType);
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&xyz.data, sizeof(icXYZNumber), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}

      /*update the final size of profile in the header*/
      tagdataptr += sizeof(icXYZNumber);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {	
	  printf("error writing file\n");
	  exit(1);
	}
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;


 
/**********************gXYZ tag******************************/


      tagdir.sig = swap((long)0x6758595aL);                 /*'gXYZ'*/
      tagdir.offset = swap((long)tagdataptr);
      size = sizeof(tagbase) + sizeof(icXYZNumber);
      printf("tag size = %d\n",size);
      tagdir.size = swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        } 



      dirptr += sizeof(tagdir);                 /*keep this pointed to the end of the 
						  tag directory*/
      tagbase.sig = swap((long)0x58595a20L);                /*'XYZ '*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }   
      tagdataptr +=sizeof(tagbase);
      xyz.data.data[0].X = double2icfixed (0.211273, icSigS15Fixed16ArrayType);
      xyz.data.data[0].Y = double2icfixed (0.526093, icSigS15Fixed16ArrayType);
      xyz.data.data[0].Z = double2icfixed (0.004135, icSigS15Fixed16ArrayType);
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&xyz.data, sizeof(icXYZNumber), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }

      /*update the final size of profile in the header*/
      tagdataptr += sizeof(icXYZNumber);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;


 
/**********************rXYZ tag******************************/


      tagdir.sig = swap((long)0x7258595aL);                 /*'rXYZ'*/
      tagdir.offset = swap((long)tagdataptr);
      size = sizeof(tagbase) + sizeof(icXYZNumber);
      printf("tag size = %d\n",size);
      tagdir.size = swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        } 



      dirptr += sizeof(tagdir);                 /*keep this pointed to the end of the 
						  tag directory*/
      tagbase.sig = swap((long)0x58595a20L);                /*'XYZ '*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }   
      tagdataptr +=sizeof(tagbase);
      xyz.data.data[0].X = double2icfixed (0.479507, icSigS15Fixed16ArrayType);
      xyz.data.data[0].Y = double2icfixed (0.278992, icSigS15Fixed16ArrayType);
      xyz.data.data[0].Z = double2icfixed (0.065887, icSigS15Fixed16ArrayType);
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&xyz.data, sizeof(icXYZNumber), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }

      /*update the final size of profile in the header*/
      tagdataptr += sizeof(icXYZNumber);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;


 
/********************whitepoint wtpt tag******************************/


      tagdir.sig = swap((long)0x77747074L);                 /*'wtpt'*/
      tagdir.offset = swap((long)tagdataptr);
      size = sizeof(tagbase) + sizeof(icXYZNumber);
      printf("tag size = %d\n",size);
      tagdir.size = swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        } 



      dirptr += sizeof(tagdir);                 /*keep this pointed to the end of the
						  tag directory*/
      tagbase.sig = swap((long)0x58595a20L);                /*'XYZ '*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }   
      tagdataptr +=sizeof(tagbase);
      xyz.data.data[0].X = double2icfixed (0.964188, icSigS15Fixed16ArrayType);
      xyz.data.data[0].Y = double2icfixed (1.000000, icSigS15Fixed16ArrayType);
      xyz.data.data[0].Z = double2icfixed (0.824875, icSigS15Fixed16ArrayType);
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&xyz.data, sizeof(icXYZNumber), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }

      /*update the final size of profile in the header*/
      tagdataptr += sizeof(icXYZNumber);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;



/*********************bTRC tag********************************/


     /*This TRC example will use the gamma option by setting count to 1 */
      tagdir.sig = swap((long)0x62545243L);			/*bTRC'*/
      tagdir.offset = swap((long)tagdataptr);
      
      size = sizeof(tagbase) + sizeof(icUInt32Number) + sizeof(icUInt16Number);
      curvealloc = (icCurveType *)calloc(size, sizeof(icUInt8Number));
      curve = &curvealloc->curve;
      printf("tag size = %d\n",size);
      tagdir.size = swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	} 
      dirptr += sizeof(tagdir);			/*keep this pointed to the end of the 
						  tag directory*/
      tagbase.sig = swap((long)0x63757276L);		/*'curv'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}   
      tagdataptr +=sizeof(icTagBase);
      curve->count = swap((long)1);
      /*Only one value, gamma, if curve count is 1 - actually a U8Fixed8Number*/
      curve->data[0] = (icUInt16Number)swap16((ushort)(2.22 * 256.0 + 0.5));
      if (fwrite(curve, size - sizeof(icTagBase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}

      /*update the final size of profile in the header*/
      tagdataptr += size - sizeof(tagbase);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {	
	  printf("error writing file\n");
	  exit(1);
	}
      free(curvealloc);
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;



/*********************gTRC tag********************************/


     /*This TRC example will use the min/max option by setting count to 2 */
      tagdir.sig = swap((long)0x67545243L);                 /*gTRC'*/
      tagdir.offset = swap((long)tagdataptr);
      
      size = sizeof(tagbase) + sizeof(icUInt32Number) + (2 *sizeof(icUInt16Number));
      curvealloc = (icCurveType *)calloc(size, sizeof(icUInt8Number));
      curve = &curvealloc->curve;
      printf("tag size = %d\n",size);
      tagdir.size = swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        } 
      dirptr += sizeof(tagdir);                 /*keep this pointed to the end of the
						  tag directory*/
      tagbase.sig = swap((long)0x63757276L);                /*'curv'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }   
      tagdataptr +=sizeof(icTagBase);
      curve->count = swap((long)2);
      /*2 values, provide min and max and cmm will interpolate */
      curve->data[0] = (icUInt16Number)swap16((ushort)0);
      curve->data[1] = (icUInt16Number)swap16((ushort)65535);
      if (fwrite(curve, size - sizeof(icTagBase), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }

      /*update the final size of profile in the header*/
      tagdataptr += size - sizeof(tagbase);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }
      free(curvealloc);
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;



/*********************rTRC tag********************************/


     /*This TRC example will use the true curve option, supplying 256 values */
      tagdir.sig = swap((long)0x72545243L);                 /*rTRC'*/
      tagdir.offset = swap((long)tagdataptr);
      
      size = sizeof(tagbase) + sizeof(icUInt32Number) + 
        (256 * sizeof(icUInt16Number));
      curvealloc = (icCurveType *)calloc(size, sizeof(icUInt8Number));
      curve = &curvealloc->curve;
      printf("tag size = %d\n",size);
      tagdir.size = swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        } 
      dirptr += sizeof(tagdir);                 /*keep this pointed to the end of the 
						  tag directory*/
      tagbase.sig = swap((long)0x63757276L);                /*'curv'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }   
      tagdataptr +=sizeof(icTagBase);
      jj = 256;
      curve->count = swap((long)jj);
      /*Supply a curve of values - create the curve using the a gamma of 2.22*/
      for (ii=0; ii<jj; ii++) {
        f32 = (float)(ii/255.0);
        curve->data[ii] = (icUInt16Number)swap16((ushort)(65535 * pow(f32, 2.22)));
      }
      if (fwrite(curve, size - sizeof(icTagBase), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }

      /*update the final size of profile in the header*/
      tagdataptr += size - sizeof(tagbase);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }
      free(curvealloc);
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;



/****************************calt, calibration data/time tag************************/


      tagdir.sig = swap((long)0x63616c74L);			/*'calt'*/
      tagdir.offset = swap((long)tagdataptr);
      size = sizeof(tagbase) + sizeof(icDateTimeNumber);
      printf("tag size = %d\n",size);
      tagdir.size = swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	} 

      dirptr += sizeof(tagdir);			/*keep this pointed to the end of the 
						  tag directory*/
      tagbase.sig = swap((long)0x6474696dL);		/*'dtim'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}   
      tagdataptr +=sizeof(icTagBase);
      /* Get the time from the system */
      clocktime = time(NULL);
      tmdatetime = localtime(&clocktime);

      datetime.date.seconds = (icUInt16Number)swap16((ushort)tmdatetime->tm_sec);
      datetime.date.minutes = (icUInt16Number)swap16((ushort)tmdatetime->tm_min);
      datetime.date.hours = (icUInt16Number)swap16((ushort)tmdatetime->tm_hour);
      datetime.date.day = (icUInt16Number)swap16((ushort)tmdatetime->tm_mday);
      datetime.date.month = (icUInt16Number)swap16((ushort)(tmdatetime->tm_mon+1));
      datetime.date.year = (icUInt16Number)swap16((ushort)(tmdatetime->tm_year + 1900));

      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&datetime.date, sizeof(icDateTimeNumber), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}

      /*update the final size of profile in the header*/
      tagdataptr += sizeof(icDateTimeType) - sizeof(icTagBase);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {	
	  printf("error writing file\n");
	  exit(1);
	}
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;




/*************************targ - characterization data target tag***************************/


      tagdir.sig = swap((long)0x74617267L);			/*targ'*/
      tagdir.offset =  swap((long)tagdataptr);
      /*The string to be entered is 18 characters, including ending space, plus base */
      size = sizeof(tagbase) + 18;
      textalloc = (icTextType *)calloc(size, sizeof(icUInt8Number));
      text = &textalloc->data;
      printf("tag size = %d\n",size);
      tagdir.size =  swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	} 
      dirptr += sizeof(tagdir);			/*keep this pointed to the end of the 
						  tag directory*/
      tagbase.sig =  swap((long)0x74657874L);		/*'text'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}   
      tagdataptr +=sizeof(icTagBase);
      strncpy(text->data,"ANSI IT8.7/1-1993",18);

      if (fwrite(text->data, 18, 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}

      /*update the final size of profile in the header*/
      tagdataptr += size - sizeof(tagbase);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {	
	  printf("error writing file\n");
	  exit(1);
	}
      free(textalloc);
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;



/**********************cprt - copyright tag*******************************/


      tagdir.sig =  swap((long)0x63707274L);			/*cprt'*/
      tagdir.offset =  swap((long)tagdataptr);
      /*The string to be entered is 34 characters, including ending space, plus base */
      size = sizeof(tagbase) + 34;
      textalloc = (icTextType *)calloc(size, sizeof(icUInt8Number));
      text = &textalloc->data;
      printf("tag size = %d\n",size);
      tagdir.size =  swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	} 
      dirptr += sizeof(tagdir);			/*keep this pointed to the end of the 
						  tag directory*/
      tagbase.sig =  swap((long)0x74657874L);		/*'text'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}   
      tagdataptr +=sizeof(icTagBase);
      strncpy(text->data,"Copyright:  Sun Microsystems 1998",34);

      if (fwrite(text, 34, 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}

      /*update the final size of profile in the header*/
      tagdataptr += size - sizeof(tagbase);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {	
	  printf("error writing file\n");
	  exit(1);
	}
      free(textalloc);
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;



/**************************dmnd - device manufacturing description tag************************/


      tagdir.sig =  swap((long)0x646d6e64L);			/*dmnd'*/
      tagdir.offset =  swap((long)tagdataptr);    
      /*tagbase + ascii count + data, UniCode code + count scriptCode code + count.
       *The Unicode desc to be entered is 48 characters */
      /*see below for detail on the size calculations*/
      size = sizeof(tagbase) +
                sizeof(icUInt32Number) + 48 +
                2*sizeof(icUInt32Number) +
                sizeof(icUInt16Number) + sizeof(icUInt8Number) + 67;

      ptr8 = (char *)calloc(130, sizeof(icUInt8Number));
      ptr8save = ptr8;
      printf("tag size = %d\n",size);
      tagdir.size =  swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	} 
      dirptr += sizeof(tagdir);			/*keep this pointed to the end of the 
						  tag directory*/
      tagbase.sig =  swap((long)0x64657363L);		/*'desc'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}   
      tagdataptr +=sizeof(icTagBase);

      /* Write the ASCII data  (48 + 4 bytes)*/
      n32 = swap(48);
      memcpy(ptr8, &n32, sizeof(icUInt32Number));
      ptr8 += sizeof(icUInt32Number);
      strncpy(ptr8,"This is the device manufacturer description tag",48); 
      ptr8 += 48;

      /* Write in the UniCode data (8 bytes minimum) */
      n32 = 0;
      memcpy(ptr8, &n32, sizeof(icUInt32Number));  /*Unicode language code */
      ptr8 += sizeof(icUInt32Number);
      memcpy(ptr8, &n32, sizeof(icUInt32Number));  /*Unicode description count*/
      ptr8 += sizeof(icUInt32Number);

      /* Write in the ScriptCode data (70 bytes minimum)*/
      n16 = swap16(0);
      n8=0;
      memcpy(ptr8, &n16, sizeof(icUInt16Number));  /*script code language code */
      ptr8 += sizeof(icUInt16Number);
      memcpy(ptr8, &n8, sizeof(icUInt8Number));  /*scriptcode count */
      ptr8 += sizeof(icUInt8Number);
      memcpy(ptr8, &n8, 67);  		    /*required 67 bytes of 0 */
      ptr8 += 67;    
      n32 = 130;
      if (fwrite(ptr8save, n32, 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}

      /*update the final size of profile in the header*/
      tagdataptr += size - sizeof(tagbase);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {	
	  printf("error writing file\n");
	  exit(1);
	}
      free(ptr8save);
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;



/******************************meas - alternative measurment tag***********************/



      tagdir.sig =  swap((long)0x6d656173L);			/*'meas'*/
      tagdir.offset =  swap((long)tagdataptr);
      size = sizeof(tagbase) + sizeof(icMeasurement);
      printf("tag size = %d\n",size);
      tagdir.size =  swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	} 


      dirptr += sizeof(tagdir);			/*keep this pointed to the end of the 
						  tag directory*/
      tagbase.sig =  swap((long)0x6d656173L);		/*'meas'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}   
      tagdataptr +=sizeof(tagbase);
      measurement.measurement.stdObserver =  swap((long)0x00000001L);  /*1931 2 degrees*/
      measurement.measurement.backing.X = 
	double2icfixed (0.72435, icSigS15Fixed16ArrayType);
      measurement.measurement.backing.Y = 
	double2icfixed (0.89567, icSigS15Fixed16ArrayType);
      measurement.measurement.backing.Z =
	double2icfixed (0.95563, icSigS15Fixed16ArrayType);
      measurement.measurement.geometry =  swap((long)0x00000001L); 	 	/*1/45 or 45/0*/
      measurement.measurement.flare =  	
	double2icfixed (0.33, icSigU16Fixed16ArrayType); 			/*30% flare*/
      measurement.measurement.illuminant =  swap((long)0x00000002L);		/*D65*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&measurement.measurement, sizeof(measurement.measurement),
		 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}

      /*update the final size of profile in the header*/
      tagdataptr += sizeof(measurement.measurement);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {	
	  printf("error writing file\n");
	  exit(1);
	}
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;


/**************************desc - profile description tag******************/


      tagdir.sig =  swap((long)0x64657363L);                 /*'desc'*/
      tagdir.offset =  swap((long)tagdataptr);    
      /*tagbase + ascii count + data, UniCode code + count scriptCode code + count.
       *The Unicode desc to be entered is 48 characters */
      /*see below for detail on the size calculations*/
      size = sizeof(tagbase) +
                sizeof(icUInt32Number) + 36 +
                2*sizeof(icUInt32Number) +
                sizeof(icUInt16Number) + sizeof(icUInt8Number) + 67;

      ptr8 = (char *)calloc(130, sizeof(icUInt8Number));
      ptr8save = ptr8;
      printf("tag size = %d\n",size);
      tagdir.size =  swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        } 
      dirptr += sizeof(tagdir);                 /*keep this pointed to the end of the          
                                                  tag directory*/
      tagbase.sig =  swap((long)0x64657363L);                /*'desc'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }   
      tagdataptr +=sizeof(icTagBase);

      /* Write the ASCII data  (48 + 4 bytes)*/
      n32 = swap(36);
      memcpy(ptr8, &n32, sizeof(icUInt32Number));
      ptr8 += sizeof(icUInt32Number);
      strncpy(ptr8,"This is the profile description tag",36); 
      ptr8 += 36;

      /* Write in the UniCode data (8 bytes minimum) */
      n32 = 0;
      memcpy(ptr8, &n32, sizeof(icUInt32Number));  /*Unicode language code */
      ptr8 += sizeof(icUInt32Number);
      memcpy(ptr8, &n32, sizeof(icUInt32Number));  /*Unicode description count*/
      ptr8 += sizeof(icUInt32Number);

      /* Write in the ScriptCode data (70 bytes minimum)*/
      n16 = swap16(0);
      n8=0;
      memcpy(ptr8, &n16, sizeof(icUInt16Number));  /*script code language code */
      ptr8 += sizeof(icUInt16Number);
      memcpy(ptr8, &n8, sizeof(icUInt8Number));  /*scriptcode count */
      ptr8 += sizeof(icUInt8Number);
      memcpy(ptr8, &n8, 67);                /*required 67 bytes of 0 */
      ptr8 += 67;    
      n32 = 130;
      if (fwrite(ptr8save, n32, 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }


      /*update the final size of profile in the header*/
      tagdataptr += size - sizeof(tagbase);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }
      free(ptr8save);
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;



/*****************pseq tag************************************/



     /*This profile sequence tag example will assume 2 profiles are linked */
      tagdir.sig =  swap((long)0x70736571L);			/*pseq'*/
      tagdir.offset =  swap((long)tagdataptr);
      /*The profile sequence tag would normally be built by a CMM upon linking two
       * or more profiles together.  This is just an example of what would be
       * written into the tag.  The size would be derived from the fields of the 
       * profiles combined.  Here it is hardcoded for 2 fictitious profiles.*/
      size = sizeof(tagbase) + sizeof(icUInt32Number) +    /*# profile descriptions*/
	2 * sizeof(icSignature) +			/*deviceMfg*/
	2 * sizeof(icSignature) +			/*deviceModel*/
	2 * sizeof(icUInt64Number) +			/*attributes*/
	2 * sizeof(icSignature) +			/*technology*/
	138 + 132 + 			/*device & model desc from profile 1*/
	138 + 132;			/*device & model desc from profile 2*/	
      descstructalloc = (icDescStruct *)calloc(size, sizeof(icUInt8Number));
      descstruct = descstructalloc;
      printf("tag size = %d\n",size);
      tagdir.size =  swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	} 
      dirptr += sizeof(tagdir);			/*keep this pointed to the end of the 
						  tag directory*/
      tagbase.sig =  swap((long)0x70736571L);		/*'pseq'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}   
      tagdataptr +=sizeof(icTagBase);
      count =  swap((long)2);
      if (fwrite(&count, sizeof(icUInt32Number), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}
      tagdataptr += sizeof(icUInt32Number);

      /*profile 1 info*/
      descstruct->deviceMfg =  swap((long)0x53554e57L);		/*SUNW  from header*/
      descstruct->deviceModel =  swap((long)0x31393938L);		/*1998 from header*/
      descstruct->attributes[1] =  swap((long)15);			/*Transparency,matte, negative, B&W*/
      descstruct->technology =  swap((long)0x43525420L);		/*CRT */

      if (fwrite(descstruct, 20, 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}
      tagdataptr +=20;

      /* text description tag type for profile 1 device */
      ptr8 = (char *)calloc(138, sizeof(icUInt8Number));
      memset(ptr8, 0, 138);
      ptr8save = ptr8;

      n32 =0x64657363L;                           /*desc*/
      memcpy(ptr8, &n32, sizeof(icUInt32Number));
      ptr8 += sizeof(icUInt32Number);
      n32 = 0;
      memcpy(ptr8, &n32, sizeof(icUInt32Number));
      ptr8 += sizeof(icUInt32Number);
      /* Write the ASCII data  (48 bytes)*/
      n32 = (icUInt32Number)swap(48);

      memcpy(ptr8, &n32, sizeof(icUInt32Number));
      ptr8 += sizeof(icUInt32Number); 
      strncpy (ptr8,"This is the device manufacturer description tag", 48);
      ptr8 += 48;

      /* Write in the UniCode data (8 bytes minimum) */
      n32 = swap((long)0);
      memcpy(ptr8, &n32, sizeof(icUInt32Number));  /*Unicode language code */
      ptr8 += sizeof(icUInt32Number);
      memcpy(ptr8, &n32, sizeof(icUInt32Number));  /*Unicode description count*/
      ptr8 += sizeof(icUInt32Number);

      /* Write in the ScriptCode data (70 bytes minimum)*/
      n16 = swap16((ushort)0);
      n8=0;
      memcpy(ptr8, &n16, sizeof(icUInt16Number));  /*script code language code */
      ptr8 += sizeof(icUInt16Number);
      memcpy(ptr8, &n8, sizeof(icUInt8Number));  /*scriptcode count */
      ptr8 += sizeof(icUInt8Number);
      memcpy(ptr8, &n8, 67);  		    /*required 67 bytes of 0 */
      ptr8 += 67;    
      n32 = 138;

      if (fwrite(ptr8save, n32, 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}
      tagdataptr += 138;
      memset(ptr8save, 0, 138);

      /* text description tag type for profile 1 model */

      ptr8 = ptr8save;
      n32 = 0x64657363L;                           /*desc*/
      memcpy(ptr8, &n32, sizeof(icUInt32Number));
      ptr8 += sizeof(icUInt32Number);
      n32 = 0;
      memcpy(ptr8, &n32, sizeof(icUInt32Number));
      ptr8 += sizeof(icUInt32Number);
      /* Write the ASCII data  (42 + 4 bytes)*/
      n32 = swap((long)42);

      memcpy(ptr8, &n32, sizeof(icUInt32Number));
      ptr8 += sizeof(icUInt32Number); 
      strncpy (ptr8,"This is the device model description tag ", 42);
      ptr8 += 42;

      /* Write in the UniCode data (8 bytes minimum) */
      n32 = swap((long)0);
      memcpy(ptr8, &n32, sizeof(icUInt32Number));  /*Unicode language code */
      ptr8 += sizeof(icUInt32Number);
      memcpy(ptr8, &n32, sizeof(icUInt32Number));  /*Unicode description count*/
      ptr8 += sizeof(icUInt32Number);

      /* Write in the ScriptCode data (70 bytes minimum)*/
      n16 = swap16((ushort)0);
      n8=0;
      memcpy(ptr8, &n16, sizeof(icUInt16Number));  /*script code language code */
      ptr8 += sizeof(icUInt16Number);
      memcpy(ptr8, &n8, sizeof(icUInt8Number));  /*scriptcode count */
      ptr8 += sizeof(icUInt8Number);
      memcpy(ptr8, &n8, 67);  		    /*required 67 bytes of 0 */
      ptr8 += 67;    
      n32 = (124 + 8);

      if (fwrite(ptr8save, n32, 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}
      tagdataptr += 132;

      memset(ptr8, 0, 138);


      /*profile 2 info*/
      descstruct->deviceMfg =  swap((long)0x53554e57L);		/*SUNW  from header*/
      descstruct->deviceModel =  swap((long)0x31393937L);		/*1997 from header*/
      descstruct->attributes[1] =  swap((long)2);			/*Reflective, matte ,positive, color*/
      descstruct->technology =  swap((long)0x696a6574L);		/*ijet */

      if (fwrite(descstruct, 20, 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}
      tagdataptr +=20;

      /* text description tag type for profile 2 device */

      ptr8 = ptr8save;
      n32 = 0x64657363L;                           /*desc*/
      memcpy(ptr8, &n32, sizeof(icUInt32Number));
      ptr8 += sizeof(icUInt32Number);
      n32 = 0;
      memcpy(ptr8, &n32, sizeof(icUInt32Number));
      ptr8 += sizeof(icUInt32Number);
      /* Write the ASCII data  (48 + 4 bytes)*/
      n32 = swap((long)48);

      memcpy(ptr8, &n32, sizeof(icUInt32Number));
      ptr8 += sizeof(icUInt32Number); 
      strncpy (ptr8,"This is the device manufacturer description tag", 48);
      ptr8 += 48;

      /* Write in the UniCode data (8 bytes minimum) */
      n32 = swap((long)0);
      memcpy(ptr8, &n32, sizeof(icUInt32Number));  /*Unicode language code */
      ptr8 += sizeof(icUInt32Number);
      memcpy(ptr8, &n32, sizeof(icUInt32Number));  /*Unicode description count*/
      ptr8 += sizeof(icUInt32Number);

      /* Write in the ScriptCode data (70 bytes minimum)*/
      n16 = swap16((ushort)0);
      n8=0;
      memcpy(ptr8, &n16, sizeof(icUInt16Number));  /*script code language code */
      ptr8 += sizeof(icUInt16Number);
      memcpy(ptr8, &n8, sizeof(icUInt8Number));  /*scriptcode count */
      ptr8 += sizeof(icUInt8Number);
      memcpy(ptr8, &n8, 67);  		    /*required 67 bytes of 0 */
      ptr8 += 67;    
      n32 = 138;

      if (fwrite(ptr8save, n32, 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}
      tagdataptr += 138;
      memset(ptr8save, 0, 138);


      /* text description tag type for profile 2 model */
      ptr8 = ptr8save;
      n32 = 0x64657363L;                           /*desc*/
      memcpy(ptr8, &n32, sizeof(icUInt32Number));
      ptr8 += sizeof(icUInt32Number);
      n32 = 0;
      memcpy(ptr8, &n32, sizeof(icUInt32Number));
      ptr8 += sizeof(icUInt32Number);
      /* Write the ASCII data  (42 + 4 bytes)*/
      n32 = swap((long)42);

      memcpy(ptr8, &n32, sizeof(icUInt32Number));
      ptr8 += sizeof(icUInt32Number);
      strncpy (ptr8,"This is the device model description tag ", 42); 
      ptr8 += 42;
      /* Write in the UniCode data (8 bytes minimum) */
      n32 = 0;
      memcpy(ptr8, &n32, sizeof(icUInt32Number));  /*Unicode language code */
      ptr8 += sizeof(icUInt32Number);
      memcpy(ptr8, &n32, sizeof(icUInt32Number));  /*Unicode description count*/
      ptr8 += sizeof(icUInt32Number);

      /* Write in the ScriptCode data (70 bytes minimum)*/
      n16 = swap16((ushort)0);
      n8=0;
      memcpy(ptr8, &n16, sizeof(icUInt16Number));  /*script code language code */
      ptr8 += sizeof(icUInt16Number);
      memcpy(ptr8, &n8, sizeof(icUInt8Number));  /*scriptcode count */
      ptr8 += sizeof(icUInt8Number);
      memcpy(ptr8, &n8, 67);  		    /*required 67 bytes of 0 */
      ptr8 += 67;    
      n32 = (124 + 8);
      if (fwrite(ptr8save, n32, 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}
      tagdataptr += 132;

      free(ptr8save);

      /*update the final size of profile in the header*/
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {	
	  printf("error writing file\n");
	  exit(1);
	}
      free(descstructalloc);
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;





/*************************scrd  Screening  description tag****************************/


      tagdir.sig =  swap((long)0x73637264L);			/*scrd'*/
      tagdir.offset =  swap((long)tagdataptr);    
      /*tagbase + ascii count + data, UniCode code + count scriptCode code + count.
       *The Unicode desc to be entered is 38 characters */
      /*see below for detail on the size calculations*/
      size = sizeof(tagbase) +
                sizeof(icUInt32Number) + 38 +
                2*sizeof(icUInt32Number) +
                sizeof(icUInt16Number) + sizeof(icUInt8Number) + 67;

      ptr8 = (char *)calloc(size - sizeof(tagbase),sizeof(icUInt8Number));
      memset(ptr8, 0, size- sizeof(tagbase));
      ptr8save = ptr8;
      printf("tag size = %d\n",size);
      tagdir.size =  swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	} 
      dirptr += sizeof(tagdir);			/*keep this pointed to the end of the 
						  tag directory*/
      tagbase.sig =  swap((long)0x64657363L);		/*'desc'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}   
      tagdataptr +=sizeof(icTagBase);

      /* Write the ASCII data  (38 + 4 bytes)*/
      n32 = swap((long)38);

      memcpy(ptr8, &n32, sizeof(icUInt32Number));
      ptr8 += sizeof(icUInt32Number);
      strncpy (ptr8,"This is the screening description tag", 38); 
      ptr8 += 38;

      /* Write in the UniCode data (8 bytes minimum) */
      n32 = swap((long)0);
      memcpy(ptr8, &n32, sizeof(icUInt32Number));  /*Unicode language code */
      ptr8 += sizeof(icUInt32Number);
      memcpy(ptr8, &n32, sizeof(icUInt32Number));  /*Unicode description count*/
      ptr8 += sizeof(icUInt32Number);

      /* Write in the ScriptCode data (70 bytes minimum)*/
      n16 = swap16((ushort)0);
      n8=0;
      memcpy(ptr8, &n16, sizeof(icUInt16Number));  /*script code language code */
      ptr8 += sizeof(icUInt16Number);
      memcpy(ptr8, &n8, sizeof(icUInt8Number));  /*scriptcode count */
      ptr8 += sizeof(icUInt8Number);
      memcpy(ptr8, &n8, 67);  		    /*required 67 bytes of 0 */
      ptr8 += 67;    
      n32 = size - sizeof(tagbase);
      if (fwrite(ptr8save, n32, 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}

      /*update the final size of profile in the header*/
      tagdataptr += size - sizeof(tagbase);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {	
	  printf("error writing file\n");
	  exit(1);
	}
      memset(ptr8save, 0, size- sizeof(tagbase));
      free(ptr8save);
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;

/**********************tech tag******************************/


      tagdir.sig = swap((long)0x74656368L);            /*'tech'*/
      tagdir.offset = swap((long)tagdataptr);
      size = sizeof(tagbase) + sizeof(icSignature);
      printf("tag size = %d\n",size);
      tagdir.size = swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        } 

      dirptr += sizeof(tagdir);         /*keep this pointed to the 
                                          end of the tag directory*/
      tagbase.sig = swap((long)0x73696720L);            /*'sig '*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }   
      tagdataptr +=sizeof(tagbase);
      sig = swap((long)0x43525420);     /*'CRT '*/
      if (fwrite(&sig, sizeof(icSignature), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }

      /*update the final size of profile in the header*/
      tagdataptr += sizeof(icSignature);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1)
 {
          printf("error writing file\n");
          exit(1);
        }

      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;

/**************************scrn  Screening tag***************************/


      tagdir.sig =  swap((long)0x7363726eL);			/*scrn'*/
      tagdir.offset =  swap((long)tagdataptr);

      size = sizeof(tagbase) +
	2 * sizeof(icUInt32Number) + 	/*# channels and screening flag*/
	6 * sizeof(icS15Fixed16Number) +  /*3 * frequency and angle */
	3 * sizeof(icUInt32Number);	/*3 * spot shape */

      screenalloc = (icScreeningType *)calloc(size, sizeof(icUInt8Number));
      memset(screenalloc, 0, size);
      screen = &screenalloc->screen;
      printf("tag size = %d\n",size);
      tagdir.size =  swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	} 
      dirptr += sizeof(tagdir);			/*keep this pointed to the end of the 
						  tag directory*/
      tagbase.sig =  swap((long)0x64617461L);		/*'data'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}   
      tagdataptr +=sizeof(icTagBase);
      jj = 3;
      screen->channels =  swap((long)jj);
      screen->screeningFlag =  swap((long)0x00000003L);	/*use printer default, lines per inch*/
      for (ii = 0; ii < jj; ii++) {
	screen->data[ii].frequency =  double2icfixed(30.56+ii, icSigS15Fixed16ArrayType);
	screen->data[ii].angle =  double2icfixed(5.489+ii, icSigS15Fixed16ArrayType);
	screen->data[ii].spotShape =  swap((long)2);			/*round*/
      }
      if (fwrite(screen, size - sizeof(tagbase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}

      /*update the final size of profile in the header*/
      tagdataptr += size - sizeof(tagbase);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {	
	  printf("error writing file\n");
	  exit(1);
	}
      memset(screenalloc, 0, size);
      free(screenalloc);
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;




/***********************bfd Under color removal and back generation tag**********************/


      tagdir.sig =  swap((long)0x62666420L);			/*bfd '*/
      tagdir.offset =  swap((long)tagdataptr);

      size = sizeof(tagbase) +
	sizeof(icUInt32Number) + 	/*# UCR curve points*/
	(6 *sizeof(icUInt16Number)) + 	/*UCR curve points (6 of them) */
	sizeof(icUInt32Number) +	/*# BG points ( 1= percentage) */
	sizeof(icUInt16Number) +	/*percentage BG	*/
	28;				/*description length*/

      ucrbgalloc = (icUcrBgType *)calloc(size, sizeof(icUInt8Number));
      memset(ucrbgalloc, 0, size);
      ucrbg = &ucrbgalloc->data;
      dataptr = (char *)ucrbg->data;
      
      printf("tag size = %d\n",size);
      tagdir.size =  swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	} 
      dirptr += sizeof(tagdir);			/*keep this pointed to the end of the 
						  tag directory*/
      tagbase.sig =  swap((long)0x64617461L);		/*'data'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}   
      tagdataptr +=sizeof(icTagBase);
      n32 = swap((long)6);
      memcpy(dataptr, &n32, sizeof(icUInt32Number));   /*number of UCR curve points*/
      dataptr += sizeof(icUInt32Number);
      for (ii=0; ii< 6; ii++) {
	n16 = swap16((ushort)(ii *(256/6)));
	memcpy(dataptr, &n16, sizeof(icUInt16Number));
	dataptr+= sizeof(icUInt16Number);
      }
      n32 =  swap((long)1);
      memcpy(dataptr, &n32, sizeof(icUInt32Number));   /*number of BG curve points*/ 
      dataptr +=  sizeof(icUInt32Number);	/* 1 means it is a percentage*/
      n16 =swap16((ushort) 94);					/* 94%*/
      memcpy(dataptr, &n16, sizeof(icUInt16Number));
      dataptr+= sizeof(icUInt16Number);
      strncpy(dataptr, "This is a UcrBg description",28);
      dataptr += 28;

      if (fwrite(ucrbg->data, size - sizeof(tagbase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}

      /*update the final size of profile in the header*/
      tagdataptr += size - sizeof(tagbase);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {	
	  printf("error writing file\n");
	  exit(1);
	}
      memset(ucrbgalloc, 0, size);
      free(ucrbgalloc);
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;




/****************************view  Viewing conditions tag*************************/


      tagdir.sig =  swap((long)0x76696577L);			/*'view'*/
      tagdir.offset =  swap((long)tagdataptr);
      size = sizeof(tagbase) + sizeof(icViewingCondition);
      printf("tag size = %d\n",size);
      tagdir.size =  swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	} 

      dirptr += sizeof(tagdir);			/*keep this pointed to the end of the 
						  tag directory*/
      tagbase.sig =  swap((long)0x76696577L);		/*'view'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}   
      tagdataptr +=sizeof(tagbase);
      view.view.illuminant.X = double2icfixed (0.279984, icSigS15Fixed16ArrayType);
      view.view.illuminant.Y = double2icfixed (0.200272, icSigS15Fixed16ArrayType);
      view.view.illuminant.Z = double2icfixed (0.840454, icSigS15Fixed16ArrayType);

      view.view.surround.X = double2icfixed (0.9856, icSigS15Fixed16ArrayType);
      view.view.surround.Y = double2icfixed (0.9777, icSigS15Fixed16ArrayType);
      view.view.surround.Z = double2icfixed (0.9712, icSigS15Fixed16ArrayType);

      view.view.stdIluminant =  swap((long)2);     /*icIlluminant65*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&view.view, size - sizeof(tagbase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}

      /*update the final size of profile in the header*/
      tagdataptr += size - sizeof(tagbase);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {	
	  printf("error writing file\n");
	  exit(1);
	}
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;



/*************************ncl2  Named color tag***************************/



      tagdir.sig =  swap((long)0x6e636c32L);			/*ncl2'*/
      tagdir.offset =  swap((long)tagdataptr);

      size = sizeof(tagbase) +
	3 * sizeof(icUInt32Number) + 	/*vendor, count, # device coords*/
	(64 *sizeof(icUInt8Number)) + 	/*2*32 - prefix, suffix arrrays */
	(64 * sizeof(icUInt8Number))+   /*root name array*/
	(12 * sizeof(icUInt16Number));	/*2 colors * 3 coords * 2 sets */

      nc2alloc = (icNamedColor2Type *)calloc(size, sizeof(icUInt8Number));
      memset(nc2alloc, 0, size);
      nc2 = &nc2alloc->ncolor;
      dataptr = (char *)nc2->data;
      
      printf("tag size = %d\n",size);
      tagdir.size =  swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	} 
      dirptr += sizeof(tagdir);			/*keep this pointed to the end of the 
						  tag directory*/
      tagbase.sig =  swap((long)0x64617461L);		/*'data'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}   
      tagdataptr +=sizeof(icTagBase);
      nc2->vendorFlag =  swap((long)0x00040000);		/*vendor specific flag-upper 16bits*/
      nc2->count =  swap((long)2);				/* number of colors described*/
      nc2->nDeviceCoords =  swap((long)3);			/* # coords for each color*/
      strncpy(nc2->prefix, "light", 32);
      strncpy(nc2->suffix, "ish", 32);
      /* color 1 */
      strncpy(dataptr, "red", 32);		/* root color name */
      dataptr += 32;
      n16 = swap16((ushort)255);
      memcpy(dataptr, &n16, sizeof(icUInt16Number));	/*the 3 PCS coords */
      dataptr += sizeof(icUInt16Number);
      n16 =  swap16((ushort)0);
      memcpy(dataptr, &n16, sizeof(icUInt16Number));
      dataptr += sizeof(icUInt16Number);
      n16 =  swap16((ushort)0);
      memcpy(dataptr, &n16, sizeof(icUInt16Number));
      dataptr += sizeof(icUInt16Number);

      n16 =  swap16((ushort)128);
      memcpy(dataptr, &n16, sizeof(icUInt16Number));	/*the 3 device coords */
      dataptr += sizeof(icUInt16Number);
      n16 =  swap16((ushort)3);
      memcpy(dataptr, &n16, sizeof(icUInt16Number));
      dataptr += sizeof(icUInt16Number);
      n16 =  swap16((ushort)4);
      memcpy(dataptr, &n16, sizeof(icUInt16Number));
      dataptr += sizeof(icUInt16Number);


      /* color 2 */
      strncpy(dataptr, "blue", 32);	/* root color name */
      dataptr += 32;
      n16 =  swap16((ushort)0);
      memcpy(dataptr, &n16, sizeof(icUInt16Number));	/*the 3 PCS coords */
      dataptr += sizeof(icUInt16Number);
      n16 =  swap16((ushort)0);
      memcpy(dataptr, &n16, sizeof(icUInt16Number));
      dataptr += sizeof(icUInt16Number);
      n16 =  swap16((ushort)255);
      memcpy(dataptr, &n16, sizeof(icUInt16Number));
      dataptr += sizeof(icUInt16Number);

      n16 =  swap16((ushort)10);
      memcpy(dataptr, &n16, sizeof(icUInt16Number));	/*the 3 device coords */
      dataptr += sizeof(icUInt16Number);
      n16 =  swap16((ushort)9);
      memcpy(dataptr, &n16, sizeof(icUInt16Number));
      dataptr += sizeof(icUInt16Number);
      n16 =  swap16((ushort)200);
      memcpy(dataptr, &n16, sizeof(icUInt16Number));
      dataptr += sizeof(icUInt16Number);


      if (fwrite(nc2, size - sizeof(tagbase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}

      /*update the final size of profile in the header*/
      tagdataptr += size - sizeof(tagbase);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {	
	  printf("error writing file\n");
	  exit(1);
	}
      memset(nc2alloc, 0, size);
      free(nc2alloc);
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;



/*************************crdi CRD info tag***************************/



      tagdir.sig =  swap((long)0x63726469L);			/*crdi'*/
      tagdir.offset = swap((long)tagdataptr);

      size = sizeof(tagbase) +
   	sizeof(icUInt32Number) +	/*count*/
	5 * sizeof(icUInt32Number) + 	/*5 sets of character counts*/
	136;			 	/*5 strings of characters totaled */

      crdalloc = (icCrdInfoType *)calloc(size, sizeof(icUInt8Number));
      crd = &crdalloc->info;
      dataptr = (char *)crd;
      printf("tag size = %d\n",size);
      tagdir.size = swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	} 
      dirptr += sizeof(tagdir);	       	/*keep this pointed to the end of the tag directory*/
      tagbase.sig = swap((long)0x63726469L);		/*'crdi'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}   
      tagdataptr +=sizeof(icTagBase);
      n32 = swap((long)24);
      memcpy(dataptr, &n32, sizeof(icUInt32Number));
      dataptr += sizeof(icUInt32Number);
      strncpy(dataptr, "PostScript product name ", 24);
      dataptr += 24;
      n32 = swap((long)28);
      memcpy(dataptr, &n32, sizeof(icUInt32Number));
      dataptr += sizeof(icUInt32Number);
      strncpy(dataptr, "Rendering intent 0 CRD name ", 28);
      dataptr += 28;
      n32 = swap((long)28);
      memcpy(dataptr, &n32, sizeof(icUInt32Number));
      dataptr += sizeof(icUInt32Number);
      strncpy(dataptr, "Rendering intent 1 CRD name ", 28);
      dataptr += 28;
      n32 = swap((long)28);
      memcpy(dataptr, &n32, sizeof(icUInt32Number));
      dataptr += sizeof(icUInt32Number);
      strncpy(dataptr, "Rendering intent 2 CRD name ", 28);
      dataptr += 28;
      n32 = swap((long)28);
      memcpy(dataptr, &n32, sizeof(icUInt32Number));
      dataptr += sizeof(icUInt32Number);
      strncpy(dataptr, "Rendering intent 3 CRD name", 28);
      dataptr += 28;

 
      if (fwrite(crd, size - sizeof(tagbase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}

      /*update the final size of profile in the header*/
      tagdataptr += size - sizeof(tagbase);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {	
	  printf("error writing file\n");
	  exit(1);
	}
      free(crdalloc);
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;



/*************************'devs'  device settings tag***************************/

      /* this sample code will include one platform with each of the 3 types of devices settings */

      tagdir.sig = swap((long)0x64657673L);                 /*devs'*/
      tagdir.offset = swap((long)tagdataptr);

      size = sizeof(tagbase) +
        1 * sizeof(icUInt32Number) +    /*number of platforms */
        3 * sizeof(icUInt32Number) +    /*platform,  size, #combinations*/
        2 * sizeof(icUInt32Number) +    /*structure size and # structures*/
        4 * sizeof(icUInt32Number) +    /*setting sample 1*/
        4 * sizeof(icUInt32Number) +    /*setting sample 2*/
        5 * sizeof(icUInt32Number);     /*setting sample 3*/

      settingstype = (icDeviceSettingsType *)calloc(size, sizeof(icUInt8Number));
      settingsdata = &settingstype->data;
    
      printf("tag size = %d\n",size);
      tagdir.size = swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        } 
      dirptr += sizeof(tagdir);                 /*keep this pointed to the end of the 
						  tag directory*/
      tagbase.sig = swap((long)0x64657673L);                /*'devs'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }   

      tagdataptr +=sizeof(icTagBase);
      ii = 0;
      settingsdata->numPlatforms = swap(1);

      settingsdata->data[ii++] = swap((long)icSigSolaris);                  /*platform*/
      settingsdata->data[ii++] = swap((long)18 * sizeof(icUInt32Number));   /* size*/ 
      settingsdata->data[ii++] = swap((long)1);                             /*combCount*/
      settingsdata->data[ii++] = swap((long)15 * sizeof(icUInt32Number));   /*structSize*/
      settingsdata->data[ii++] = swap((long)3);                             /*numStructs*/

      /* setting sample 1 - media*/
      settingsdata->data[ii++] = swap((long)icSigMedia);                    /*settingSig*/
      settingsdata->data[ii++] = swap((long)sizeof(icUInt32Number));        /*size*/
      settingsdata->data[ii++] = swap((long)1);                             /*numSettings*/
      settingsdata->data[ii++] = swap((long)icTrans);                       /*tranparency*/
      /* setting sample 2 - halftone*/
      settingsdata->data[ii++] = swap((long)icSigHalftone);                 /*settingSig*/
      settingsdata->data[ii++] = swap((long)sizeof(icUInt32Number));        /*size*/
      settingsdata->data[ii++] = swap((long)1);                             /*numSettings*/
      settingsdata->data[ii++] = swap((long)icErrorDiffusion);              /*dithering*/

      /* setting sample 3 - resolution*/
      settingsdata->data[ii++] = swap((long)icSigResolution);               /*settingSig*/
      settingsdata->data[ii++] = swap((long)sizeof(icUInt64Number));        /*size*/
      settingsdata->data[ii++] = swap((long)1);                             /*numSettings*/
      settingsdata->data[ii++] = swap((long)600);                           /*dpi, y resolution*/
      settingsdata->data[ii++] = swap((long)400);                           /*dpi, x resolution*/


      if (fwrite(settingsdata, size- sizeof(tagbase), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }

      /*update the final size of profile in the header*/
      tagdataptr += size - sizeof(tagbase);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }
      free(settingstype);

      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;


/*************************'resp'  response curve tag***************************/

      /* this sample code will include 2 types of measurement set for 3 channels */

      tagdir.sig = swap((long)0x72657370L);                 /*'resp'*/
      tagdir.offset = swap((long)tagdataptr);

      size = sizeof(tagbase) +
        2 * sizeof(icUInt16Number) +    /*# channels, #measurement types*/
        2 * sizeof(icUInt32Number) +    /*offset to each type's data */
        4 * sizeof(icUInt32Number) +    /*signature, # measurements for each type*/
        3 * sizeof(icXYZNumber) +       /*type 1: 3 measurements for type 1*/   
        3 * sizeof(icResponse16Number) +
        4 * sizeof(icResponse16Number) +
        2 * sizeof(icResponse16Number) +
        4 * sizeof(icUInt32Number) +    /*signature, # measurements for each typel*/   
        3 * sizeof(icXYZNumber) +       /*type 2: 3 measurements for type 2*/   
        2 * sizeof(icResponse16Number) +
        2 * sizeof(icResponse16Number) +
        2 * sizeof(icResponse16Number);
  
      respcurvetype = (icResponseCurveSet16Type *)calloc(size, sizeof(icUInt8Number));
      response = (icResponse *)calloc (size - (4* sizeof(icUInt32Number)),sizeof(icUInt8Number));
      response = &respcurvetype->data;

      printf("tag size = %d\n",size);
      tagdir.size = swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        } 
      dirptr += sizeof(tagdir);                 /*keep this pointed to the end of the 
						  tag directory*/
      tagbase.sig = swap((long)0x72637332L);                /*'rcs2'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }   

      tagdataptr +=sizeof(icTagBase);

      response->channels = swap16((ushort)3);
      response->numTypes = swap16((ushort)2);
      i = 0;
      response->data[i++] = swap((long)20); /*byte offset to first measurement type data*/
      jj = i++;                 /*save this to enter offset for second type*/
      /*measurement type 1*/
      response->data[i++] = swap((long)icDN);               /*  measurement type */
      response->data[i++] = swap((long)3);          /*# measurements for channel 1 */
      response->data[i++] = swap((long)4);          /*# measurements for channel 2 */
      response->data[i++] = swap((long)2);          /*# measurements for channel 3 */

      response->data[i++] = double2icfixed (0.53678, icSigS15Fixed16ArrayType); /* X */
      response->data[i++] = double2icfixed (0.84736, icSigS15Fixed16ArrayType); /* Y */
      response->data[i++] = double2icfixed (0.48475, icSigS15Fixed16ArrayType); /* Z */
      response->data[i++] = double2icfixed (0.98676, icSigS15Fixed16ArrayType); /* X */
      response->data[i++] = double2icfixed (0.77658, icSigS15Fixed16ArrayType); /* Y */
      response->data[i++] = double2icfixed (0.99867, icSigS15Fixed16ArrayType); /* Z */

      response->data[i++] = double2icfixed (0.09374, icSigS15Fixed16ArrayType); /* X */
      response->data[i++] = double2icfixed (0.04746, icSigS15Fixed16ArrayType); /* Y */
      response->data[i++] = double2icfixed (0.04958, icSigS15Fixed16ArrayType); /* Z */

      response->data[i++] = swap16((ushort)0x00010000);                                         /*interval*/
      response->data[i++] = double2icfixed (0.12, icSigS15Fixed16ArrayType);    /*measurement*/ 
      response->data[i++] = swap(0x00020000);                                         /*interval*/
      response->data[i++] = double2icfixed (0.34, icSigS15Fixed16ArrayType);    /*measurement*/ 
      response->data[i++] = swap(0x00030000);                                         /*interval*/
      response->data[i++] = double2icfixed (0.56, icSigS15Fixed16ArrayType);    /*measurement*/ 

      response->data[i++] = swap(0x00040000);                                         /*interval*/
      response->data[i++] = double2icfixed (0.78, icSigS15Fixed16ArrayType);    /*measurement*/ 
      response->data[i++] = swap(0x00050000);                                         /*interval*/
      response->data[i++] = double2icfixed (0.91, icSigS15Fixed16ArrayType);    /*measurement*/ 
      response->data[i++] = swap(0x00060000);                                         /*interval*/
      response->data[i++] = double2icfixed (0.23, icSigS15Fixed16ArrayType);    /*measurement*/ 
      response->data[i++] = swap(0x00070000);                                         /*interval*/
      response->data[i++] = double2icfixed (0.45, icSigS15Fixed16ArrayType);    /*measurement*/ 

      response->data[i++] = swap(0x00080000);                                         /*interval*/
      response->data[i++] = double2icfixed (0.67, icSigS15Fixed16ArrayType);    /*measurement*/ 
      response->data[i++] = swap(0x00090000);                                         /*interval*/
      response->data[i++] = double2icfixed (0.89, icSigS15Fixed16ArrayType);    /*measurement*/ 


      response->data[jj] = swap((long)(4 *i + 12));      /*byte offset to second measurement type data*/
      /*measurement type 2*/
      response->data[i++] = swap((long)icStaT);             /*  measurement type */
      response->data[i++] = swap((long)2);          /*# measurements for channel 1 */
      response->data[i++] = swap((long)2);          /*# measurements for channel 2 */
      response->data[i++] = swap((long)2);          /*# measurements for channel 3 */

      response->data[i++] = double2icfixed (0.0, icSigS15Fixed16ArrayType);     /* X */
      response->data[i++] = double2icfixed (0.0, icSigS15Fixed16ArrayType);     /* Y */
      response->data[i++] = double2icfixed (0.0, icSigS15Fixed16ArrayType);     /* Z */
      response->data[i++] = double2icfixed (0.76678, icSigS15Fixed16ArrayType); /* X */
      response->data[i++] = double2icfixed (0.84736, icSigS15Fixed16ArrayType); /* Y */
      response->data[i++] = double2icfixed (0.98475, icSigS15Fixed16ArrayType); /* Z */

      response->data[i++] = double2icfixed (0.23676, icSigS15Fixed16ArrayType); /* X */
      response->data[i++] = double2icfixed (0.34658, icSigS15Fixed16ArrayType); /* Y */
      response->data[i++] = double2icfixed (0.32867, icSigS15Fixed16ArrayType); /* Z */

      response->data[i++] = swap(0x000a0000);                                         /*interval*/
      response->data[i++] = double2icfixed (0.98, icSigS15Fixed16ArrayType);    /*measurement*/ 
      response->data[i++] = swap(0x000b0000);                                         /*interval*/
      response->data[i++] = double2icfixed (0.76, icSigS15Fixed16ArrayType);    /*measurement*/ 

      response->data[i++] = swap(0x000c0000);                                         /*interval*/
      response->data[i++] = double2icfixed (0.54, icSigS15Fixed16ArrayType);    /*measurement*/ 
      response->data[i++] = swap(0x000d0000);                                         /*interval*/
      response->data[i++] = double2icfixed (0.32, icSigS15Fixed16ArrayType);    /*measurement*/ 
 
      response->data[i++] = swap(0x000e0000);                                         /*interval*/
      response->data[i++] = double2icfixed (0.19, icSigS15Fixed16ArrayType);    /*measurement*/ 
      response->data[i++] = swap(0x000f0000);                                         /*interval*/
      response->data[i++] = double2icfixed (0.87, icSigS15Fixed16ArrayType);    /*measurement*/ 


      if (fwrite(response, size- sizeof(tagbase), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }

      /*update the final size of profile in the header*/
      tagdataptr += size - sizeof(tagbase);
      fseek(fd, 0, SEEK_SET);
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {
          printf("error writing file\n");
          exit(1);
        }

      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;


/**************************chrm Chromaticity tag***************************/

      tagdir.sig = swap((long)0x6368726dL);			/*'chrm'*/
      tagdir.offset = swap((long)tagdataptr);

      size = sizeof(tagbase) +
	2 * sizeof(icUInt16Number) + 	/*# channels and type */
	6 * sizeof(icU16Fixed16Number);  /* 3 channels of xy coordinates */

      chromaalloc = (icChromaticityType *)calloc(size, sizeof(icUInt8Number));
      memset(chromaalloc, 0, size);
      chroma = &chromaalloc->chromaticity;
      ptr8save = (char *)chroma;
      printf("tag size = %d\n",size);
      tagdir.size = swap((long)size);
      fseek(fd, dirptr, SEEK_SET);
      if (fwrite(&tagdir, sizeof(icTag), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	} 
      dirptr += sizeof(tagdir);			/*keep this pointed to the end of the 
						  tag directory*/
      tagbase.sig = swap((long)0x6368726dL);		/*'chrm'*/
      fseek(fd, tagdataptr, SEEK_SET);
      if (fwrite(&tagbase, sizeof(icTagBase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}   
      tagdataptr +=sizeof(icTagBase);
      chroma->channels = (icUInt16Number)swap16((ushort)3);	/* number of channels */
      chroma->type = (icUInt16Number)swap16((ushort)2);		/*SMPTE RP145-1994*/
       /* the xy coordinates are defined int he specification*/
      chroma->data[0] = (icU16Fixed16Number)double2icfixed(0.64, 
			icSigU16Fixed16ArrayType);
      chroma->data[1] = (icU16Fixed16Number)double2icfixed(0.33, 
                        icSigU16Fixed16ArrayType);
      chroma->data[2] = (icU16Fixed16Number)double2icfixed(0.29, 
                        icSigU16Fixed16ArrayType);
      chroma->data[3] = (icU16Fixed16Number)double2icfixed(0.60, 
                        icSigU16Fixed16ArrayType);
      chroma->data[4] = (icU16Fixed16Number)double2icfixed(0.15, 
                        icSigU16Fixed16ArrayType);
      chroma->data[5] = (icU16Fixed16Number)double2icfixed(0.06, 
                        icSigU16Fixed16ArrayType);
      if (fwrite(ptr8save, size - sizeof(tagbase), 1, fd) != 1) {
	  printf("error writing file\n");
	  exit(1);
	}

      /*update the final size of profile in the header*/
      tagdataptr += size - sizeof(tagbase);
      fseek(fd, 0, SEEK_SET);  
      tempdataptr = swap((long)tagdataptr);
      if (fwrite(&tempdataptr, sizeof(icUInt32Number), 1, fd) != 1) {	
	  printf("error writing file\n");
	  exit(1);
	}
      memset(chromaalloc, 0, size);
      free(chromaalloc);
      /* May need add to the tagdataptr to make sure the next tag 
         lands on a four byte boundary
         */
      divt = div(tagdataptr, 4);
      tagdataptr += divt.rem;


      exit(0);
    }
}




/******************Support Functions*********************/

void print_luttag(FILE *fd, icTag tag, 
		  icUInt32Number verbose, char buf[5])
{
    icLut16Type         *lut16alloc;
    icLut16             *lut16;
    icLut8Type          *lut8alloc;
    icLut8              *lut8;
    long                swap(long);
    ushort		swap16(ushort);
    icSignature         sig;
    icInt32Number	i, ii, ii2, j, jj, jj2,  size, perrow, dirptr;
    div_t		divt;

	printf("signature = 0x%x signatureId = %s, offset = %d size = %d\n",
		tag.sig, buf, tag.offset, tag.size);
	fseek(fd, (long) tag.offset, 0);
	if (fread(&sig, 4, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	sig = swap(sig);
	fseek(fd, (long) tag.offset, 0);  	/*reset */
	if (sig == 0x6d667431L) {		/* mft1 = 8 bit lut */
        printf("8 bit lut type\n");
	lut8alloc = (icLut8Type *)calloc(tag.size, sizeof(icUInt8Number));
	if (fread(lut8alloc, tag.size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	lut8 = &lut8alloc->lut;
        printf("Lut 8 type\n");
        printf("%d  ", (short)lut8->inputChan);
        printf("%d  ", (short)lut8->outputChan);
        printf("%d\n", (short)lut8->clutPoints);
        printf("%f ", icfixed2double((long)lut8->e00, icSigS15Fixed16ArrayType));
        printf("%f ", icfixed2double((long)lut8->e01, icSigS15Fixed16ArrayType));
        printf("%f\n", icfixed2double((long)lut8->e02, icSigS15Fixed16ArrayType));
        printf("%f ", icfixed2double((long)lut8->e10, icSigS15Fixed16ArrayType));
        printf("%f ", icfixed2double((long)lut8->e11, icSigS15Fixed16ArrayType));
        printf("%f\n", icfixed2double((long)lut8->e12, icSigS15Fixed16ArrayType));
        printf("%f ", icfixed2double((long)lut8->e20, icSigS15Fixed16ArrayType));
        printf("%f ", icfixed2double((long)lut8->e21, icSigS15Fixed16ArrayType));
        printf("%f\n", icfixed2double((long)lut8->e22, icSigS15Fixed16ArrayType));

	if (!verbose) printf("tables will not be dumped unless you use -v option\n");

	if(verbose) {
	/*print the input table*/
	j = lut8->inputChan * 256;
        printf("input tables: #channels * 256 = %d\n", j);
	perrow = 9;
	dirptr = 0;
	divt = div(j, perrow);
	for (ii=0; ii<divt.quot; ii++) {
	  for (ii2 =0; ii2<perrow; ii2++) {
	     printf("%d  ",lut8->data[dirptr + (ii*perrow) + ii2]);
	  }
	  printf("\n");
	}
	for (ii2 =j - divt.rem; ii2<j; ii2++) {
	  printf("%d  ",lut8->data[ii2]);
	}
	printf("\n");	
	dirptr += j;

	/*print the clut*/
	printf("\n");
	j = (pow(lut8->clutPoints,lut8->inputChan)
		  *lut8->outputChan) ;
	printf("clut: #clut points**#input channels * #output channels = %d\n", 
	       j);
	divt = div(j, perrow);
	for (ii=0; ii<divt.quot; ii++) {
	  for (ii2 =0; ii2<perrow; ii2++) {
	     printf("%d  ",lut8->data[dirptr + (ii*perrow) + ii2]);
	  }
	  printf("\n");
	}
	for (ii2 =dirptr + j - divt.rem; ii2<dirptr + j; ii2++) {
	  printf("%d  ",lut8->data[ii2]);
	}
	printf("\n");	
	dirptr += j;

	/*print the output table*/
	j = (lut8->outputChan * 256);
        printf("output tables: #channels * 256 = %d\n", j);	

	divt = div(j, perrow);
	for (ii=0; ii<divt.quot; ii++) {
	  for (ii2 =0; ii2<perrow; ii2++) {
	     printf("%d  ",lut8->data[dirptr + (ii*perrow) + ii2]);
	  }
	  printf("\n");
	}
	for (ii2 =dirptr +j - divt.rem; ii2<dirptr + j; ii2++) {
	  printf("%d  ",lut8->data[ii2]);
	}
	printf("\n");	
	dirptr += j;

	}  /*end if verbose*/

	free(lut8alloc);

	} else 
	if (sig == 0x6d667432L) { 		/* mft2 = 16 bit lut */
        printf("16 bit lut type\n");
	lut16alloc = (icLut16Type *)calloc(tag.size, sizeof(icUInt8Number));
	if (fread(lut16alloc, tag.size, 1, fd) != 1) {
	  printf("error reading file\n");
	  exit(1);
	}
	lut16 = &lut16alloc->lut;
        printf("Lut 16 type\n");
        printf("%d  ", (short)lut16->inputChan);
        printf("%d  ", (short)lut16->outputChan);
        printf("%d\n", (short)lut16->clutPoints);
        printf("%f ", icfixed2double((long)lut16->e00, icSigS15Fixed16ArrayType));
        printf("%f ", icfixed2double((long)lut16->e01, icSigS15Fixed16ArrayType));
        printf("%f\n", icfixed2double((long)lut16->e02, icSigS15Fixed16ArrayType));
        printf("%f ", icfixed2double((long)lut16->e10, icSigS15Fixed16ArrayType));
        printf("%f ", icfixed2double((long)lut16->e11, icSigS15Fixed16ArrayType));
        printf("%f\n", icfixed2double((long)lut16->e12, icSigS15Fixed16ArrayType));
        printf("%f ", icfixed2double((long)lut16->e20, icSigS15Fixed16ArrayType));
        printf("%f ", icfixed2double((long)lut16->e21, icSigS15Fixed16ArrayType));
        printf("%f\n", icfixed2double((long)lut16->e22, icSigS15Fixed16ArrayType));
        printf("%d   %d \n", swap16((ushort)lut16->inputEnt),
                swap16((ushort)lut16->outputEnt));

	if (!verbose) printf("tables will not be dumped unless you use -v option\n");

	if(verbose) {
	/*print the input table*/
	j = lut16->inputChan * swap16((ushort)lut16->inputEnt);
        printf("input tables: #channels * #input entries = %d\n", j);
	perrow = 9;
	dirptr = 0;
	divt = div(j, perrow);
	for (ii=0; ii<divt.quot; ii++) {
	  for (ii2 =0; ii2<perrow; ii2++) {
	     printf("%d  ",swap16((ushort)lut16->data[dirptr + (ii*perrow) + ii2]));
	  }
	  printf("\n");
	}
	for (ii2 =j - divt.rem; ii2<j; ii2++) {
	  printf("%d  ",lut16->data[ii2]);
	}
	printf("\n");	
	dirptr += j;

	/*print the clut*/
	printf("\n");
	j = (pow(lut16->clutPoints,lut16->inputChan)
		  *lut16->outputChan) ;
	printf("clut: #clut points**#input channels * #output channels = %d\n", 
	       j);
	divt = div(j, perrow);
	for (ii=0; ii<divt.quot; ii++) {
	  for (ii2 =0; ii2<perrow; ii2++) {
	     printf("%d  ",lut16->data[dirptr + (ii*perrow) + ii2]);
	  }
	  printf("\n");
	}
	for (ii2 =dirptr + j - divt.rem; ii2<dirptr + j; ii2++) {
	  printf("%d  ",lut16->data[ii2]);
	}
	printf("\n");	
	dirptr += j;

	/*print the output table*/
	j = (swap16((ushort)lut16->outputEnt)*lut16->outputChan);
        printf("output tables: #channels * #output entries = %d\n", j);	

	divt = div(j, perrow);
	for (ii=0; ii<divt.quot; ii++) {
	  for (ii2 =0; ii2<perrow; ii2++) {
	     printf("%d  ",swap16((ushort)lut16->data[dirptr + (ii*perrow) + ii2]));
	  }
	  printf("\n");
	}
	for (ii2 =dirptr +j - divt.rem; ii2<dirptr + j; ii2++) {
	  printf("%d  ",swap16((ushort)lut16->data[ii2]));
	}
	printf("\n");	
	dirptr += j;

	}  /*end if verbose*/

	free(lut16alloc);
	}
        printf("\n\n");

}
/*
 * Routine to byte swap a long, just returns on big-endian
 */
long
swap(long value)
{
#ifdef _LITTLE_ENDIAN
    char	*ptr, c;

    ptr = (char *)&value;
    c = *(ptr+1);
    *(ptr+1) = *(ptr+2);
    *(ptr+2) = c;
    c = *ptr;
    *ptr = *(ptr+3);
    *(ptr+3) = c;
#endif _LITTLE_ENDIAN

    return(value);
}

/*
 * Routine to byte swap a short, just returns on big-endian
 */
ushort
swap16(ushort value)
{
#ifdef _LITTLE_ENDIAN
    char	*ptr, c;

    ptr = (char *)&value;
    c = *(ptr+1);
    *(ptr+1) = *(ptr);
    *ptr = c;

#endif _LITTLE_ENDIAN

    return(value);
}


/* Convert an 8.8 unsigned fixed type to double */
double
icfixed2gamma(icUInt16Number val)
{
    double      retval;
    ushort 	a, b;
    
    val = swap16((ushort)val);
    retval = (double) (val/256.0);
    return (retval);
}

/*
 * Convert IC fixed point to a double
 */
double
icfixed2double(long val, long type)
{
    double              retval;
    val = swap((long)val);
    switch(type) {
    case icSigS15Fixed16ArrayType :
        retval = (double) (val / 65536.0);
        break;
    case icSigU16Fixed16ArrayType :
        retval = (double)(val/65536.0);
        break;
    default :
        retval  = -1.0;
        break;
    }

    return(retval);
}
/*
 * Convert double to an IC fixed point number
 * is not accurate enough for boundary conditions
 */
long
double2icfixed(double val, long type)
{
    long                retval;
    short               a;
    ushort     		b, c;

    switch(type) {
    case icSigS15Fixed16ArrayType :
        a = (short)(val);
        b = (ushort)((val - a) * 65536.0);
        retval = (long) ((a <<16) | b);
        break;
    case icSigU16Fixed16ArrayType :
        c = (ushort)(val);
        b = (ushort)((val - c) * 65536.0);
        retval = (unsigned long)((c <<16) | b);
        break;
    default :
        retval  = -1.0;
        break;
    }
    retval = swap((long)retval);
    return(retval);
}

